﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using com.epson.bank.driver;

namespace TM_S9000Sample
{
    public partial class TMS9000SampleForm : Form
    {
        private CApp            m_objDriverControl  = null;
        private StructByStep    m_objConfigData     = null;

        private delegate void CallbackProcDisplayMicrText(ErrorCode errResult, CMicrResult objMicrResult);
        private delegate void CallbackProcSaveMicrText(ErrorCode errResult, CMicrResult objMicrResult);
        private delegate void CallbackProcDisplayImage(ErrorCode errResult, CImageResult objImageResult, bool bFourSheetScan);
        private delegate void CallbackProcSaveImageData(ErrorCode errResult, CImageResult objImageResult);

        CallbackProcDisplayMicrText         CallbackFuncDisplayMicrText     = null;
        CallbackProcSaveMicrText            CallbackFuncSaveMicrText        = null;
        CallbackProcDisplayImage            CallbackFuncDisplayImageData    = null;
        CallbackProcSaveImageData           CallbackFuncSaveImageData       = null;

        CApp.CallbackProcProcessError       CallbackFuncProcessError        = null;
        CApp.CallbackProcImage              CallbackFuncImage               = null;
        CApp.CallbackProcMicr               CallbackFuncMicr                = null;
        CApp.CallbackProcPhysicalEndorse    CallbackFuncPhysicalEndorse     = null;
        CApp.CallbackProcElectronicEndorse  CallbackFuncElectronicEndorse   = null;

        private bool m_bScanCancelError = false;

        private Bitmap m_biImage2Front      = null;
        private Bitmap m_biImage2Back       = null;
        private Bitmap m_biImage4RGBFront   = null;
        private Bitmap m_biImage4RGBBack    = null;
        private Bitmap m_biImage4IrFront    = null;
        private Bitmap m_biImage4IrBack     = null;

        public TMS9000SampleForm()
        {
            InitializeComponent();
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                if (components != null)
                {
                    components.Dispose();
                }
                // end process
                ErrorCode errRet = ErrorCode.SUCCESS;

                try
                {
                    errRet = TerminateDriver();
                    if (errRet != ErrorCode.SUCCESS)
                    {
                        ShowErrorMessage(errRet);
                    }
                }
                catch
                {
                    base.Dispose(disposing);
                }
            }
            base.Dispose(disposing);
        }

        private void TMS9000SampleForm_Load(object sender, EventArgs e)
        {
            m_objDriverControl = new CApp();
            m_objConfigData = new StructByStep();

            if (InitializeDriver() != true)
            {
                this.Close();
                return;
            }

            // Initialize data
            InitializeConfigData();

            // Initialize controls
            LoadResourceString();
            UpdateControls();
        }
        
        /////////////////////////////////////////////////////////////////
        // Procedure of DriveControl Callback
        /////////////////////////////////////////////////////////////////

        ////////////////
        //// Step 1
        ////////////////
        private void CallbackProcProcessError(ErrorCode errResult)
        {
	        if( m_objDriverControl == null )
	        {
		        return;
	        }

            ASB eStatus = new ASB();
            // Display message box correspondent to error
            switch (errResult)
            {
                case ErrorCode.ERR_COVER_OPEN:
                    do
                    {
                        MessageBox.Show(ConstComStr.MSG_01_001, ConstComStr.CAPTION_01_001, MessageBoxButtons.OK, MessageBoxIcon.Error);
                        m_objDriverControl.GetDeviceStatus(ref eStatus);

                    }while( (eStatus & ASB.ASB_COVER_OPEN) == ASB.ASB_COVER_OPEN );
                    m_objDriverControl.CancelError();
                    break;

                case ErrorCode.ERR_PAPER_JAM:
                    if (m_objConfigData.nRadio_ScanningMedia == ConstComVal.VAL_CONFIGDLG_RADIO_SM_CHECKPAPER)
                    {
                        do
                        {
                            MessageBox.Show(ConstComStr.MSG_02_000, ConstComStr.CAPTION_02_000, MessageBoxButtons.OK, MessageBoxIcon.Error);
                            m_objDriverControl.GetDeviceStatus(ref eStatus);

                        } while ( ((eStatus & ASB.ASB_EJECT_SENSOR_NO_PAPER) != ASB.ASB_EJECT_SENSOR_NO_PAPER)
                                        || ((eStatus & ASB.ASB_SLIP_PAPER_SIZE) != ASB.ASB_SLIP_PAPER_SIZE)
                                        || ((eStatus & ASB.ASB_PAPER_INTERMEDIATE) != ASB.ASB_PAPER_INTERMEDIATE));
                        m_objDriverControl.CancelError();
                    }
                    else
                    {
                        MessageBox.Show(ConstComStr.MSG_02_000, ConstComStr.CAPTION_02_000, MessageBoxButtons.OK, MessageBoxIcon.Error);
                        m_objDriverControl.CancelError();
                    }
                    break;

                case ErrorCode.ERR_MICR_BADDATA:
                case ErrorCode.ERR_MICR_NODATA:
                    break;

                default:
                    ShowErrorMessage(errResult);
                    break;
            }
        }

        ////////////////
        //// Step 2
        ////////////////
        private void CallbackProcImage()
        {
	        if( m_objDriverControl == null )
	        {
		        return;
	        }
            // Configure display data for each scanning side and light source
            if (m_objConfigData.nRadio_ImageChannel == ConstComVal.VAL_CONFIGDLG_RADIO_IC_RGB)
            {
                // Configure 2 plane data for RGB image
                UpdateImageData(ScanSide.MF_SCAN_FACE_FRONT, ImageType.MF_SCAN_IMAGE_VISIBLE, false);
                UpdateImageData(ScanSide.MF_SCAN_FACE_BACK, ImageType.MF_SCAN_IMAGE_VISIBLE, false);

                // Save image if necessary
                if (m_objConfigData.bCheck_Scan_SaveFile == ConstComVal.VAL_CONFIGDLG_CHECK_MI_SAVEFILE_ON)
                {
                    SaveImageData(ScanSide.MF_SCAN_FACE_FRONT, ImageType.MF_SCAN_IMAGE_VISIBLE);
                    SaveImageData(ScanSide.MF_SCAN_FACE_BACK, ImageType.MF_SCAN_IMAGE_VISIBLE);
                }
            }
            else if (m_objConfigData.nRadio_ImageChannel == ConstComVal.VAL_CONFIGDLG_RADIO_IC_IR)
            {
                // Configure 2 plane data for IR image
                UpdateImageData(ScanSide.MF_SCAN_FACE_FRONT, ImageType.MF_SCAN_IMAGE_INFRARED, false);
                UpdateImageData(ScanSide.MF_SCAN_FACE_BACK, ImageType.MF_SCAN_IMAGE_INFRARED, false);

                // Save image if necessary
                if (m_objConfigData.bCheck_Scan_SaveFile == ConstComVal.VAL_CONFIGDLG_CHECK_MI_SAVEFILE_ON)
                {
                    SaveImageData(ScanSide.MF_SCAN_FACE_FRONT, ImageType.MF_SCAN_IMAGE_INFRARED);
                    SaveImageData(ScanSide.MF_SCAN_FACE_BACK, ImageType.MF_SCAN_IMAGE_INFRARED);
                }
            }
            else if (m_objConfigData.nRadio_ImageChannel == ConstComVal.VAL_CONFIGDLG_RADIO_IC_RGBIR)
            {
                // Configure 4 plane image
                UpdateImageData(ScanSide.MF_SCAN_FACE_FRONT, ImageType.MF_SCAN_IMAGE_VISIBLE, true);
                UpdateImageData(ScanSide.MF_SCAN_FACE_BACK, ImageType.MF_SCAN_IMAGE_VISIBLE, true);
                UpdateImageData(ScanSide.MF_SCAN_FACE_FRONT, ImageType.MF_SCAN_IMAGE_INFRARED, true);
                UpdateImageData(ScanSide.MF_SCAN_FACE_BACK, ImageType.MF_SCAN_IMAGE_INFRARED, true);

                // Save image if necessary
                if (m_objConfigData.bCheck_Scan_SaveFile == ConstComVal.VAL_CONFIGDLG_CHECK_MI_SAVEFILE_ON)
                {
                    SaveImageData(ScanSide.MF_SCAN_FACE_FRONT, ImageType.MF_SCAN_IMAGE_VISIBLE);
                    SaveImageData(ScanSide.MF_SCAN_FACE_BACK, ImageType.MF_SCAN_IMAGE_VISIBLE);
                    SaveImageData(ScanSide.MF_SCAN_FACE_FRONT, ImageType.MF_SCAN_IMAGE_INFRARED);
                    SaveImageData(ScanSide.MF_SCAN_FACE_BACK, ImageType.MF_SCAN_IMAGE_INFRARED);
                }
            }
        }

        ////////////////
        //// Step 3
        ////////////////
        private void CallbackProcMicr()
        {
            // Obtain MICR data
            CMicrResult cMicrResult = new CMicrResult();
            ErrorCode errResult = m_objDriverControl.GetMicr(ref cMicrResult);
            CallbackFuncDisplayMicrText(errResult, cMicrResult);

            // Save MICR data if necessary
            if (m_objConfigData.bCheck_MICR_SaveFile == ConstComVal.VAL_CONFIGDLG_CHECK_MI_SAVEFILE_ON)
            {
                CallbackSaveMicrText(errResult, cMicrResult);
            }
        }

        private void CallbackDisplayMicrText(ErrorCode errResult, CMicrResult cMicrResult)
        {
            if (this.InvokeRequired)
            {
                this.BeginInvoke(CallbackFuncDisplayMicrText, new object[] { errResult, cMicrResult });
            }
            else
            {
                string strMicrRet = null;

                if (errResult == ErrorCode.SUCCESS)
                {
                    edtMicrText.Text = cMicrResult.GetMicrStr();
                }
                else
                {
                    strMicrRet = GetErrorString(errResult);
                    edtMicrText.Text = strMicrRet;
                }
            }
        }

        ////////////////
        //// Step 4
        ////////////////
        private void CallbackProcPhysicalEndorse(uint unTransactionNumber)
        {
            ErrorCode errResult = ErrorCode.SUCCESS;

            // Specify endorsement text
            if (m_objDriverControl != null)
            {
                errResult = m_objDriverControl.PhysicalEndorseFunction(unTransactionNumber);
                if (errResult != ErrorCode.SUCCESS)
                {
                    ShowErrorMessage(errResult);
                }
            }
        }

        private void CallbackProcElectronicEndorse()
        {
            ErrorCode errResult = ErrorCode.SUCCESS;

            // Specify endorsement text
            if (m_objDriverControl != null)
            {
                errResult = m_objDriverControl.ElectronicEndorseFunction();
                if (errResult != ErrorCode.SUCCESS)
                {
                    ShowErrorMessage(errResult);
                }
            }
        }
        
        /////////////////////////////////////////////////////////////////
        // CTMS9000SampleDlg message handlers
        /////////////////////////////////////////////////////////////////
        private void stImage2Front_Paint(object sender, System.Windows.Forms.PaintEventArgs e)
        {
            if(stImage2Front.Image != null)
			{
                stImage2Front.Image.Dispose();
			}

            if (m_biImage2Front != null)
            {
                DrawImage(stImage2Front, m_biImage2Front);
            }
        }

        private void stImage2Back_Paint(object sender, System.Windows.Forms.PaintEventArgs e)
        {
            if (stImage2Back.Image != null)
            {
                stImage2Back.Image.Dispose();
            }

            if (m_biImage2Back != null)
            {
                DrawImage(stImage2Back, m_biImage2Back);
            }
        }

        private void stImage4RGBFront_Paint(object sender, System.Windows.Forms.PaintEventArgs e)
        {
            if (stImage4RGBFront.Image != null)
            {
                stImage4RGBFront.Image.Dispose();
            }

            if (m_biImage4RGBFront != null)
            {
                DrawImage(stImage4RGBFront, m_biImage4RGBFront);
            }
        }

        private void stImage4RGBBack_Paint(object sender, System.Windows.Forms.PaintEventArgs e)
        {
            if (stImage4RGBBack.Image != null)
            {
                stImage4RGBBack.Image.Dispose();
            }

            if (m_biImage4RGBBack != null)
            {
                DrawImage(stImage4RGBBack, m_biImage4RGBBack);
            }
        }

        private void stImage4IRFront_Paint(object sender, System.Windows.Forms.PaintEventArgs e)
        {
            if (stImage4IRFront.Image != null)
            {
                stImage4IRFront.Image.Dispose();
            }

            if (m_biImage4IrFront != null)
            {
                DrawImage(stImage4IRFront, m_biImage4IrFront);
            }
        }

        private void stImage4IRBack_Paint(object sender, System.Windows.Forms.PaintEventArgs e)
        {
            if (stImage4IRBack.Image != null)
            {
                stImage4IRBack.Image.Dispose();
            }

            if (m_biImage4IrBack != null)
            {
                DrawImage(stImage4IRBack, m_biImage4IrBack);
            }
        }

        /////////////////////////////////////////////////////////////////
        // Button Event
        /////////////////////////////////////////////////////////////////

        ////////////////
        //// Step 1
        ////////////////
        private void btnScan_Click(object sender, EventArgs e)
        {
            if (m_objDriverControl == null)
            {
                return;
            }

            // Reset controls
            ////////////////
            //// Step 2
            ////////////////
            ClearImage();
            ////////////////
            //// Step 3
            ////////////////
            edtMicrText.Text = "";

            // Scanning
            if (m_objConfigData.nRadio_ScanningMedia == ConstComVal.VAL_CONFIGDLG_RADIO_SM_CHECKPAPER)
            {
                ErrorCode errRet = m_objDriverControl.ScanCheck();
                if (errRet != ErrorCode.SUCCESS)
                {
                    ShowErrorMessage(errRet);
                    m_objDriverControl.CancelError();
                    return;
                }
            }
            else if (m_objConfigData.nRadio_ScanningMedia == ConstComVal.VAL_CONFIGDLG_RADIO_SM_CARD)
            {
                ErrorCode errRet = m_objDriverControl.ScanCard();
                if (errRet != ErrorCode.SUCCESS)
                {
                    ShowErrorMessage(errRet);
                    m_objDriverControl.CancelError();
                    return;
                }
            }
        }

        private void btnExit_Click(object sender, EventArgs e)
        {
            ErrorCode errRet = ErrorCode.SUCCESS;

            errRet = m_objDriverControl.ExitDevice();
            if (errRet != ErrorCode.SUCCESS)
            {
                ShowErrorMessage(errRet);
            }
            Close();
        }

        private void btnScanCancel_Click(object sender, EventArgs e)
        {
            if (m_objDriverControl == null)
            {
                return;
            }

            ErrorCode errRet = m_objDriverControl.CancelScan();
            if (errRet != ErrorCode.SUCCESS)
            {
                m_bScanCancelError = true;
                ShowErrorMessage(errRet);
                m_objDriverControl.CancelError();
            }

        }

        ////////////////
        //// Step 2
        ////////////////

        private void btnConfig_Click(object sender, EventArgs e)
        {
            ConfigureForm cConfigureDlg = new ConfigureForm();

            cConfigureDlg.SetConfigData(m_objConfigData);

            if (cConfigureDlg.ShowDialog(this) == DialogResult.OK)
            {
                cConfigureDlg.GetConfigData(ref m_objConfigData);

                ////////////////
                //// Step 2
                ////////////////
                CScanParam cScanParam = new CScanParam();
                ConvertConfigDataToScanParam(ref m_objConfigData, ref cScanParam);
                if (m_objDriverControl != null)
                {
                    m_objDriverControl.SetScanParam(ref cScanParam);
                }

                ////////////////
                //// Step 3
                ////////////////
                CMicrParam cMicrParam = new CMicrParam();
                ConvertConfigDataToMicrParam(ref m_objConfigData, ref cMicrParam);
                if (m_objDriverControl != null)
                {
                    m_objDriverControl.SetMicrParam(ref cMicrParam);
                }

                ////////////////
                //// Step 4
                ////////////////
                CEndorseParam cEndorseParam = new CEndorseParam();
                ConvertConfigDataToEndorseParam(ref m_objConfigData, ref cEndorseParam);
                if (m_objDriverControl != null)
                {
                    m_objDriverControl.SetEndorseParam(ref cEndorseParam);
                }

                COptionParam cOptionParam = new COptionParam();
                ConvertConfigDataToOptionParam(ref m_objConfigData, ref cOptionParam);
                if (m_objDriverControl != null)
                {
                    m_objDriverControl.SetOptionParam(ref cOptionParam);
                }

                UpdateControls();
            }
        }

        ////////////////
        //// Step 3
        ////////////////
        private void btnMicrCleaning_Click(object sender, EventArgs e)
        {
            if (m_objDriverControl == null)
            {
                return;
            }

            ErrorCode errRet = m_objDriverControl.CleaningMicr();
            if (errRet != ErrorCode.SUCCESS)
            {
                ShowErrorMessage(errRet);
                m_objDriverControl.CancelError();
            }
        }

        /////////////////////////////////////////////////////////////////
        // Member functions
        /////////////////////////////////////////////////////////////////

        private bool InitializeDriver()
        {
            ErrorCode errResult = ErrorCode.SUCCESS;
            DialogResult drRet = 0;
	        string strResult = null;

	        if( m_objDriverControl == null )
	        {
		        return false;
	        }

	        // Open device and register callback functions
            errResult = m_objDriverControl.InitDevice();
            while (errResult != ErrorCode.SUCCESS)
	        {
                if (errResult == ErrorCode.ERR_UNKNOWN)
                {
                    drRet = MessageBox.Show(ConstComStr.MSG_06_000, ConstComStr.CAPTION_06_000, MessageBoxButtons.OKCancel, MessageBoxIcon.Error);
                }
                else
                {
                    strResult = GetErrorString(errResult);
                    drRet = MessageBox.Show(ConstComStr.MSG_00_000, strResult, MessageBoxButtons.OKCancel, MessageBoxIcon.Error);
                }

                if (drRet == DialogResult.OK)
		        {
                    errResult = m_objDriverControl.InitDevice();
		        }
		        else
		        {
			        return false;
		        }
	        }

            errResult = m_objDriverControl.LoadEndorseFile();
            switch ( errResult )
            {
                case ErrorCode.SUCCESS:
                    break;

                case ErrorCode.ERR_NOT_FOUND:
                    MessageBox.Show(ConstComStr.MSG_00_001, ConstComStr.CAPTION_03_042, MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return false;

                case ErrorCode.ERR_IMAGE_FILEOPEN:
                    MessageBox.Show(ConstComStr.MSG_00_002, ConstComStr.CAPTION_03_039, MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return false;

                case ErrorCode.ERR_IMAGE_FILEREAD:
                    MessageBox.Show(ConstComStr.MSG_00_002, ConstComStr.CAPTION_03_049, MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return false;

                default:
                    ShowErrorMessage(errResult);
                    return false;
            }

            CallbackFuncDisplayMicrText     = new CallbackProcDisplayMicrText(CallbackDisplayMicrText);
            CallbackFuncSaveMicrText        = new CallbackProcSaveMicrText(CallbackSaveMicrText);
            CallbackFuncDisplayImageData    = new CallbackProcDisplayImage(CallbackDisplayImageData);
            CallbackFuncSaveImageData       = new CallbackProcSaveImageData(CallbackSaveImageData);

            CallbackFuncProcessError        = new CApp.CallbackProcProcessError(CallbackProcProcessError);
            CallbackFuncImage               = new CApp.CallbackProcImage(CallbackProcImage);
            CallbackFuncMicr                = new CApp.CallbackProcMicr(CallbackProcMicr);
            CallbackFuncPhysicalEndorse     = new CApp.CallbackProcPhysicalEndorse(CallbackProcPhysicalEndorse);
            CallbackFuncElectronicEndorse   = new CApp.CallbackProcElectronicEndorse(CallbackProcElectronicEndorse);

            // Specify callback function from DriverControl
            m_objDriverControl.SetProcessErrorCallback(CallbackFuncProcessError);
            m_objDriverControl.SetImageCallback(CallbackFuncImage);
            m_objDriverControl.SetMicrCallback(CallbackFuncMicr);
	        //Step4
            m_objDriverControl.SetPhysicalEndorseCallback(CallbackFuncPhysicalEndorse);
            m_objDriverControl.SetElectronicEndorseCallback(CallbackFuncElectronicEndorse);

	        return true;
        }

        private ErrorCode TerminateDriver()
        {
            ErrorCode errRet = ErrorCode.SUCCESS;

	        if( m_objDriverControl == null )
	        {
                return ErrorCode.ERR_NO_MEMORY;
	        }

            errRet = m_objDriverControl.ExitDevice();
            if (errRet != ErrorCode.SUCCESS)
	        {
                return errRet;
	        }

            return errRet;
        }

        private void ShowErrorMessage(ErrorCode errResult)
        {
            switch (errResult)
            {
                case ErrorCode.ERR_TYPE:
                    MessageBox.Show(ConstComStr.MSG_03_000, ConstComStr.CAPTION_03_000, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_OPENED:
                    MessageBox.Show(ConstComStr.MSG_03_001, ConstComStr.CAPTION_03_001, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_NO_PRINTER:
                    MessageBox.Show(ConstComStr.MSG_03_002, ConstComStr.CAPTION_03_002, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_NO_TARGET:
                    MessageBox.Show(ConstComStr.MSG_03_003, ConstComStr.CAPTION_03_003, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_NO_MEMORY:
                    MessageBox.Show(ConstComStr.MSG_03_004, ConstComStr.CAPTION_03_004, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_HANDLE:
                    MessageBox.Show(ConstComStr.MSG_03_005, ConstComStr.CAPTION_03_005, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_TIMEOUT:
                    MessageBox.Show(ConstComStr.MSG_03_006, ConstComStr.CAPTION_03_006, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_ACCESS:
                    MessageBox.Show(ConstComStr.MSG_03_007, ConstComStr.CAPTION_03_007, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_PARAM:
                    MessageBox.Show(ConstComStr.MSG_03_008, ConstComStr.CAPTION_03_008, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_NOT_SUPPORT:
                    MessageBox.Show(ConstComStr.MSG_03_009, ConstComStr.CAPTION_03_009, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_OFFLINE:
                    MessageBox.Show(ConstComStr.MSG_03_010, ConstComStr.CAPTION_03_010, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_NOT_EPSON:
                    MessageBox.Show(ConstComStr.MSG_03_011, ConstComStr.CAPTION_03_011, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_WITHOUT_CB:
                    if (m_bScanCancelError == true)
                    {
                        MessageBox.Show(ConstComStr.MSG_03_012_01, ConstComStr.CAPTION_03_012, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                        m_bScanCancelError = false;
                    }
                    else
                    {
                        MessageBox.Show(ConstComStr.MSG_03_012_00, ConstComStr.CAPTION_03_012, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    }
                    break;
                case ErrorCode.ERR_BUFFER_OVER_FLOW:
                    MessageBox.Show(ConstComStr.MSG_03_013, ConstComStr.CAPTION_03_013, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_REGISTRY:
                    MessageBox.Show(ConstComStr.MSG_03_014, ConstComStr.CAPTION_03_014, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_PAPERINSERT_TIMEOUT:
                    MessageBox.Show(ConstComStr.MSG_03_015, ConstComStr.CAPTION_03_015, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_EXEC_FUNCTION:
                    MessageBox.Show(ConstComStr.MSG_03_016, ConstComStr.CAPTION_03_016, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_EXEC_MICR:
                    MessageBox.Show(ConstComStr.MSG_03_017, ConstComStr.CAPTION_03_017, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_EXEC_SCAN:
                    MessageBox.Show(ConstComStr.MSG_03_018, ConstComStr.CAPTION_03_018, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_RESET:
                    MessageBox.Show(ConstComStr.MSG_03_019, ConstComStr.CAPTION_03_019, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_ABORT:
                    MessageBox.Show(ConstComStr.MSG_03_020, ConstComStr.CAPTION_03_020, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_MICR:
                    MessageBox.Show(ConstComStr.MSG_03_021, ConstComStr.CAPTION_03_021, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_SCAN:
                    MessageBox.Show(ConstComStr.MSG_03_022, ConstComStr.CAPTION_03_022, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_LINE_OVERFLOW:
                    MessageBox.Show(ConstComStr.MSG_03_023, ConstComStr.CAPTION_03_023, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_PAPER_PILED:
                    MessageBox.Show(ConstComStr.MSG_03_024, ConstComStr.CAPTION_03_024, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_PAPER_JAM:
                    MessageBox.Show(ConstComStr.MSG_03_025, ConstComStr.CAPTION_03_025, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_COVER_OPEN:
                    MessageBox.Show(ConstComStr.MSG_03_026, ConstComStr.CAPTION_03_026, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_MICR_NODATA:
                    MessageBox.Show(ConstComStr.MSG_03_027, ConstComStr.CAPTION_03_027, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_MICR_BADDATA:
                    MessageBox.Show(ConstComStr.MSG_03_028, ConstComStr.CAPTION_03_028, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_MICR_NOISE:
                    MessageBox.Show(ConstComStr.MSG_03_029, ConstComStr.CAPTION_03_029, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_SCN_COMPRESS:
                    MessageBox.Show(ConstComStr.MSG_03_030, ConstComStr.CAPTION_03_030, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_PAPER_EXIST:
                    MessageBox.Show(ConstComStr.MSG_03_031, ConstComStr.CAPTION_03_031, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_PAPER_INSERT:
                    MessageBox.Show(ConstComStr.MSG_03_032, ConstComStr.CAPTION_03_032, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_EXEC_SCAN_CHECK_CONTINUOUS:
                    MessageBox.Show(ConstComStr.MSG_03_033, ConstComStr.CAPTION_03_033, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_EXEC_SCAN_CHECK_ONEBYONE:
                    MessageBox.Show(ConstComStr.MSG_03_034, ConstComStr.CAPTION_03_034, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_EXEC_SCAN_IDCARD:
                    MessageBox.Show(ConstComStr.MSG_03_035, ConstComStr.CAPTION_03_035, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_EXEC_PRINT_ROLLPAPER:
                    MessageBox.Show(ConstComStr.MSG_03_036, ConstComStr.CAPTION_03_036, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_EXEC_PRINT_VALIDATION:
                    MessageBox.Show(ConstComStr.MSG_03_037, ConstComStr.CAPTION_03_037, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_THREAD:
                    MessageBox.Show(ConstComStr.MSG_03_038, ConstComStr.CAPTION_03_038, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_IMAGE_FILEOPEN:
                    MessageBox.Show(ConstComStr.MSG_03_039, ConstComStr.CAPTION_03_039, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_IMAGE_UNKNOWNFORMAT:
                    MessageBox.Show(ConstComStr.MSG_03_040, ConstComStr.CAPTION_03_040, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_SIZE:
                    MessageBox.Show(ConstComStr.MSG_03_041, ConstComStr.CAPTION_03_041, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_NOT_FOUND:
                    MessageBox.Show(ConstComStr.MSG_03_042, ConstComStr.CAPTION_03_042, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_NOT_EXEC:
                    MessageBox.Show(ConstComStr.MSG_03_043, ConstComStr.CAPTION_03_043, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_BARCODE_NODATA:
                    MessageBox.Show(ConstComStr.MSG_03_044, ConstComStr.CAPTION_03_044, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_MICR_PARSE:
                    MessageBox.Show(ConstComStr.MSG_03_045, ConstComStr.CAPTION_03_045, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_SCN_IQA:
                    MessageBox.Show(ConstComStr.MSG_03_046, ConstComStr.CAPTION_03_046, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_PRINT_DATA_LENGTH_EXCEED:
                    MessageBox.Show(ConstComStr.MSG_03_047, ConstComStr.CAPTION_03_047, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_PRINT_DATA_UNRECEIVE:
                    MessageBox.Show(ConstComStr.MSG_03_048, ConstComStr.CAPTION_03_048, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ERR_IMAGE_FILEREAD:
                    MessageBox.Show(ConstComStr.MSG_03_049, ConstComStr.CAPTION_03_049, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                default:
                    MessageBox.Show(errResult.ToString(), "ERR", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
            }
        }

        private string GetErrorString(ErrorCode errResultCode)
        {
            switch (errResultCode)
            {

                case ErrorCode.SUCCESS:
                    return ConstComStr.CAPTION_00_000;
                case ErrorCode.ERR_TYPE:
                    return ConstComStr.CAPTION_03_000;
                case ErrorCode.ERR_OPENED:
                    return ConstComStr.CAPTION_03_001;
                case ErrorCode.ERR_NO_PRINTER:
                    return ConstComStr.CAPTION_03_002;
                case ErrorCode.ERR_NO_TARGET:
                    return ConstComStr.CAPTION_03_003;
                case ErrorCode.ERR_NO_MEMORY:
                    return ConstComStr.CAPTION_03_004;
                case ErrorCode.ERR_HANDLE:
                    return ConstComStr.CAPTION_03_005;
                case ErrorCode.ERR_TIMEOUT:
                    return ConstComStr.CAPTION_03_006;
                case ErrorCode.ERR_ACCESS:
                    return ConstComStr.CAPTION_03_007;
                case ErrorCode.ERR_PARAM:
                    return ConstComStr.CAPTION_03_008;
                case ErrorCode.ERR_NOT_SUPPORT:
                    return ConstComStr.CAPTION_03_009;
                case ErrorCode.ERR_OFFLINE:
                    return ConstComStr.CAPTION_03_010;
                case ErrorCode.ERR_NOT_EPSON:
                    return ConstComStr.CAPTION_03_011;
                case ErrorCode.ERR_WITHOUT_CB:
                    return ConstComStr.CAPTION_03_012;
                case ErrorCode.ERR_BUFFER_OVER_FLOW:
                    return ConstComStr.CAPTION_03_013;
                case ErrorCode.ERR_REGISTRY:
                    return ConstComStr.CAPTION_03_014;
                case ErrorCode.ERR_PAPERINSERT_TIMEOUT:
                    return ConstComStr.CAPTION_03_015;
                case ErrorCode.ERR_EXEC_FUNCTION:
                    return ConstComStr.CAPTION_03_016;
                case ErrorCode.ERR_EXEC_MICR:
                    return ConstComStr.CAPTION_03_017;
                case ErrorCode.ERR_EXEC_SCAN:
                    return ConstComStr.CAPTION_03_018;
                case ErrorCode.ERR_RESET:
                    return ConstComStr.CAPTION_03_019;
                case ErrorCode.ERR_ABORT:
                    return ConstComStr.CAPTION_03_020;
                case ErrorCode.ERR_MICR:
                    return ConstComStr.CAPTION_03_021;
                case ErrorCode.ERR_SCAN:
                    return ConstComStr.CAPTION_03_022;
                case ErrorCode.ERR_LINE_OVERFLOW:
                    return ConstComStr.CAPTION_03_023;
                case ErrorCode.ERR_PAPER_PILED:
                    return ConstComStr.CAPTION_03_024;
                case ErrorCode.ERR_PAPER_JAM:
                    return ConstComStr.CAPTION_03_025;
                case ErrorCode.ERR_COVER_OPEN:
                    return ConstComStr.CAPTION_03_026;
                case ErrorCode.ERR_MICR_NODATA:
                    return ConstComStr.CAPTION_03_027;
                case ErrorCode.ERR_MICR_BADDATA:
                    return ConstComStr.CAPTION_03_028;
                case ErrorCode.ERR_MICR_NOISE:
                    return ConstComStr.CAPTION_03_029;
                case ErrorCode.ERR_SCN_COMPRESS:
                    return ConstComStr.CAPTION_03_030;
                case ErrorCode.ERR_PAPER_EXIST:
                    return ConstComStr.CAPTION_03_031;
                case ErrorCode.ERR_PAPER_INSERT:
                    return ConstComStr.CAPTION_03_032;
                case ErrorCode.ERR_EXEC_SCAN_CHECK_CONTINUOUS:
                    return ConstComStr.CAPTION_03_033;
                case ErrorCode.ERR_EXEC_SCAN_CHECK_ONEBYONE:
                    return ConstComStr.CAPTION_03_034;
                case ErrorCode.ERR_EXEC_SCAN_IDCARD:
                    return ConstComStr.CAPTION_03_035;
                case ErrorCode.ERR_EXEC_PRINT_ROLLPAPER:
                    return ConstComStr.CAPTION_03_036;
                case ErrorCode.ERR_EXEC_PRINT_VALIDATION:
                    return ConstComStr.CAPTION_03_037;
                case ErrorCode.ERR_THREAD:
                    return ConstComStr.CAPTION_03_038;
                case ErrorCode.ERR_IMAGE_FILEOPEN:
                    return ConstComStr.CAPTION_03_039;
                case ErrorCode.ERR_IMAGE_UNKNOWNFORMAT:
                    return ConstComStr.CAPTION_03_040;
                case ErrorCode.ERR_SIZE:
                    return ConstComStr.CAPTION_03_041;
                case ErrorCode.ERR_NOT_FOUND:
                    return ConstComStr.CAPTION_03_042;
                case ErrorCode.ERR_NOT_EXEC:
                    return ConstComStr.CAPTION_03_043;
                case ErrorCode.ERR_BARCODE_NODATA:
                    return ConstComStr.CAPTION_03_044;
                case ErrorCode.ERR_MICR_PARSE:
                    return ConstComStr.CAPTION_03_045;
                case ErrorCode.ERR_SCN_IQA:
                    return ConstComStr.CAPTION_03_046;
                case ErrorCode.ERR_PRINT_DATA_LENGTH_EXCEED:
                    return ConstComStr.CAPTION_03_047;
                case ErrorCode.ERR_PRINT_DATA_UNRECEIVE:
                    return ConstComStr.CAPTION_03_048;
                case ErrorCode.ERR_IMAGE_FILEREAD:
                    return ConstComStr.CAPTION_03_049;
                default:
                    return errResultCode.ToString();
            }
        }

        private void InitializeConfigData()
        {
	        if( m_objConfigData == null )
	        {
		        return;
	        }

	        ////////////////
	        //// Step 2 ////
	        ////////////////

            CScanParam cScanParam = new CScanParam();
            m_objDriverControl.GetScanParam(ref cScanParam);
            ConvertScanParamToConfigData(ref cScanParam, ref m_objConfigData);

            ////////////////
            //// Step 3 ////
            ////////////////

            CMicrParam cMicrParam = new CMicrParam();
            m_objDriverControl.GetMicrParam(ref cMicrParam);
            ConvertMicrParamToConfigData(ref cMicrParam, ref m_objConfigData);

            ////////////////
            //// Step 4 ////
            ////////////////

            CEndorseParam cEndorseParam = new CEndorseParam();
            m_objDriverControl.GetEndorseParam(ref cEndorseParam);
            ConvertEndorseParamToConfigData(ref cEndorseParam, ref m_objConfigData);

            COptionParam cOptionParam = new COptionParam();
            m_objDriverControl.GetOptionParam(ref cOptionParam);
            ConvertOptionParamToConfigData(ref cOptionParam, ref m_objConfigData);
        }

        private void LoadResourceString()
        {
	        ////////////////
	        //// Step 1 ////
	        ////////////////

	        // Title
	        this.Text               = ConstComStr.STR_SAMPLEDLG_TITLE;

	        // Scan Button
	        btnScan.Text            = ConstComStr.STR_SAMPLEDLG_BUTTON_SCAN;

	        // ScanCancel Button
	        btnScanCancel.Text      = ConstComStr.STR_SAMPLEDLG_BUTTON_SCANCANCEL;

	        // Exit Button
	        btnExit.Text            = ConstComStr.STR_SAMPLEDLG_BUTTON_EXIT;


	        ////////////////
	        //// Step 2 ////
	        ////////////////

	        // Config Button
	        btnConfig.Text          = ConstComStr.STR_SAMPLEDLG_BUTTON_CONFIG;


	        ////////////////
	        //// Step 3 ////
	        ////////////////

	        // MICR Text Static
	        stMicrText.Text         = ConstComStr.STR_SAMPLEDLG_STATIC_MICRTEXT;

	        // Micr Cleaning Button
	        btnMicrCleaning.Text    = ConstComStr.STR_SAMPLEDLG_BUTTON_MICRCLEANING;

        }

        private void UpdateControls()
        {
	        ////////////////
	        //// Step 2 ////
	        ////////////////
	        if( m_objConfigData.nRadio_ImageChannel == ConstComVal.VAL_CONFIGDLG_RADIO_IC_RGBIR )
	        {
		        stImage2Front.Visible       = false;
		        stImage2Back.Visible        = false;
		        stImage4RGBFront.Visible    = true;
		        stImage4RGBBack.Visible     = true;
		        stImage4IRBack.Visible      = true;
		        stImage4IRFront.Visible     = true;
	        }
	        else
	        {
		        stImage2Front.Visible       = true;
		        stImage2Back.Visible        = true;
		        stImage4RGBFront.Visible    = false;
		        stImage4RGBBack.Visible     = false;
                stImage4IRBack.Visible      = false;
                stImage4IRFront.Visible     = false;
	        }
        }

        private void ConvertConfigDataToScanParam( ref StructByStep objConfigData, ref CScanParam objScanParam )
        {
            if (objScanParam == null || objConfigData == null)
	        {
		        return;
	        }

            switch (objConfigData.nRadio_ScanningMedia)
	        {
		        case ConstComVal.VAL_CONFIGDLG_RADIO_SM_CHECKPAPER:
                    objScanParam.SetScanMedia(ScanUnit.EPS_BI_SCN_UNIT_CHECKPAPER);
			        break;
		        case ConstComVal.VAL_CONFIGDLG_RADIO_SM_CARD:
                    objScanParam.SetScanMedia(ScanUnit.EPS_BI_SCN_UNIT_CARD);
			        break;
		        default:
			        break;
	        }

            switch (objConfigData.nRadio_ImageChannel)	// ScanGradation
	        {
		        case ConstComVal.VAL_CONFIGDLG_RADIO_IC_RGB:
                    if (objConfigData.nRadio_RGBColorDepth == ConstComVal.VAL_CONFIGDLG_RADIO_RGBCD_COLOR)
			        {
                        objScanParam.SetImageChannel(ImageTypeOption.EPS_BI_SCN_OPTION_COLOR);
			        }
			        else
			        {
                        objScanParam.SetImageChannel(ImageTypeOption.EPS_BI_SCN_OPTION_GRAYSCALE);
			        }
			        break;
		        case ConstComVal.VAL_CONFIGDLG_RADIO_IC_IR:
                    objScanParam.SetImageChannel(ImageTypeOption.EPS_BI_SCN_OPTION_IR);
			        break;
		        case ConstComVal.VAL_CONFIGDLG_RADIO_IC_RGBIR:
                    if (objConfigData.nRadio_RGBColorDepth == ConstComVal.VAL_CONFIGDLG_RADIO_RGBCD_COLOR)
			        {
                        objScanParam.SetImageChannel(ImageTypeOption.EPS_BI_SCN_OPTION_COLOR_IR);
			        }
			        else
			        {
                        objScanParam.SetImageChannel(ImageTypeOption.EPS_BI_SCN_OPTION_GRAYSCALE_IR);
			        }
			        break;
		        default:
			        break;
	        }

            switch (objConfigData.nRadio_RGBColorDepth)	// ScanGradation
	        {
		        case ConstComVal.VAL_CONFIGDLG_RADIO_RGBCD_COLOR:
                    objScanParam.SetRGBColorDepth(com.epson.bank.driver.ColorDepth.EPS_BI_SCN_24BIT);
			        break;
		        case ConstComVal.VAL_CONFIGDLG_RADIO_RGBCD_GRAY:
                    objScanParam.SetRGBColorDepth(com.epson.bank.driver.ColorDepth.EPS_BI_SCN_8BIT);
			        break;
		        case ConstComVal.VAL_CONFIGDLG_RADIO_RGBCD_BW:
                    objScanParam.SetRGBColorDepth(com.epson.bank.driver.ColorDepth.EPS_BI_SCN_1BIT);
			        break;
		        default:
			        break;
	        }

            switch (objConfigData.nRadio_IRColorDepth)	// ScanGradation
	        {
		        case ConstComVal.VAL_CONFIGDLG_RADIO_IRCD_GRAY:
                    objScanParam.SetIRColorDepth(com.epson.bank.driver.ColorDepth.EPS_BI_SCN_8BIT);
			        break;
		        case ConstComVal.VAL_CONFIGDLG_RADIO_IRCD_BW:
                    objScanParam.SetIRColorDepth(com.epson.bank.driver.ColorDepth.EPS_BI_SCN_1BIT);
			        break;
		        default:
			        break;
	        }

            switch (objConfigData.nCombo_Resolution)	// ScanGradation
	        {
		        case ConstComVal.VAL_CONFIGDLG_COMBO_RE_600600:
                    objScanParam.SetResolution(MfScanDpi.MF_SCAN_DPI_600);
			        break;
		        case ConstComVal.VAL_CONFIGDLG_COMBO_RE_300300:
                    objScanParam.SetResolution(MfScanDpi.MF_SCAN_DPI_300);
			        break;
		        case ConstComVal.VAL_CONFIGDLG_COMBO_RE_240240:
                    objScanParam.SetResolution(MfScanDpi.MF_SCAN_DPI_240);
			        break;
		        case ConstComVal.VAL_CONFIGDLG_COMBO_RE_200200:
                    objScanParam.SetResolution(MfScanDpi.MF_SCAN_DPI_200);
			        break;
		        case ConstComVal.VAL_CONFIGDLG_COMBO_RE_120120:
                    objScanParam.SetResolution(MfScanDpi.MF_SCAN_DPI_120);
			        break;
		        case ConstComVal.VAL_CONFIGDLG_COMBO_RE_100100:
                    objScanParam.SetResolution(MfScanDpi.MF_SCAN_DPI_100);
			        break;
		        default:
			        break;
	        }
        }

        private void ConvertConfigDataToMicrParam( ref StructByStep objConfigData, ref CMicrParam objMicrParam )
        {
	        if( objMicrParam == null || objConfigData == null )
	        {
		        return;
	        }

	        switch( objConfigData.nRadio_MICRFont )	
	        {
		        case ConstComVal.VAL_CONFIGDLG_RADIO_MF_E13B:
			        objMicrParam.SetFont( MfMicrFont.MF_MICR_FONT_E13B );
			        break;
		        case ConstComVal.VAL_CONFIGDLG_RADIO_MF_CMC7:
                    objMicrParam.SetFont(MfMicrFont.MF_MICR_FONT_CMC7);
			        break;
		        default:
			        break;
	        }

	        switch( objConfigData.bCheck_MICR_ClearSpace )
	        {
		        case ConstComVal.VAL_CONFIGDLG_CHECK_MI_CLEARSPACE_ON:
			        objMicrParam.SetClearSpace( true );
			        break;
		        case ConstComVal.VAL_CONFIGDLG_CHECK_MI_CLEARSPACE_OFF:
			        objMicrParam.SetClearSpace( false );
			        break;
		        default:
			        break;
	        }

	        switch( objConfigData.bCheck_MICR_SaveFile )
	        {
		        case ConstComVal.VAL_CONFIGDLG_CHECK_MI_SAVEFILE_ON:
			        objMicrParam.SetParsing( true );
			        break;
		        case ConstComVal.VAL_CONFIGDLG_CHECK_MI_SAVEFILE_OFF:
			        objMicrParam.SetParsing( false );
			        break;
		        default:
			        break;
	        }
        }

        ////////////////
        //// Step 4
        ////////////////
        private void ConvertConfigDataToEndorseParam( ref StructByStep objConfigData, ref CEndorseParam objEndorseParam )
        {
	        if( objEndorseParam == null || objConfigData == null )
	        {
		        return;
	        }

	        switch( objConfigData.nRadio_PrintType)
	        {
		        case ConstComVal.VAL_CONFIGDLG_RADIO_PT_PHYSICAL:
                    objEndorseParam.SetEndorseType(PrintingStation.MF_ST_PHYSICAL_ENDORSEMENT);
			        break;
		        case ConstComVal.VAL_CONFIGDLG_RADIO_PT_ELECTRONIC:
                    objEndorseParam.SetEndorseType(PrintingStation.MF_ST_E_ENDORSEMENT);
			        break;
		        default:
			        break;
	        }

	        switch( objConfigData.bCheck_Endorsement_PrintText )	// ScanGradation
	        {
		        case ConstComVal.VAL_CONFIGDLG_CHECK_EN_PRINTTEXT_ON:
			        objEndorseParam.SetPrintText( true );
			        break;
		        case ConstComVal.VAL_CONFIGDLG_CHECK_EN_PRINTTEXT_OFF:
			        objEndorseParam.SetPrintText( false );
			        break;
		        default:
			        break;
	        }

	        switch( objConfigData.bCheck_Endorsement_PrintImage )	// ScanGradation
	        {
		        case ConstComVal.VAL_CONFIGDLG_CHECK_EN_PRINTIMAGE_ON:
			        objEndorseParam.SetPrintImage( true );
			        break;
		        case ConstComVal.VAL_CONFIGDLG_CHECK_EN_PRINTIMAGE_OFF:
			        objEndorseParam.SetPrintImage( false );
			        break;
		        default:
			        break;
	        }
        }

        private void ConvertConfigDataToOptionParam(ref StructByStep objConfigData, ref COptionParam objOptionParam)
        {
            if (objOptionParam == null || objConfigData == null)
            {
                return;
            }

            switch (objConfigData.nRadio_PrintMode)
	        {
                case ConstComVal.VAL_CONFIGDLG_RADIO_PM_HIGHSPEEDMODE:
                    objOptionParam.SetEndorsePrintMode(MfEndorsePrintMode.MF_ENDORSEPRINT_MODE_HIGHSPEED);
			        break;
                case ConstComVal.VAL_CONFIGDLG_RADIO_PM_DATAWAITINGMODE:
                    objOptionParam.SetEndorsePrintMode(MfEndorsePrintMode.MF_ENDORSEPRINT_MODE_DATAWAITING);
			        break;
		        default:
			        break;
	        }

            switch (objConfigData.bCheck_Endorsement_PrnDataUnreceiveCancel)
	        {
                case ConstComVal.VAL_CONFIGDLG_CHECK_EN_PRNDATAUNRECEIVECANCEL_ON:
                    objOptionParam.SetPrnDataUnreceiveCancel(MfCancel.MF_CANCEL_ENABLE);
			        break;
                case ConstComVal.VAL_CONFIGDLG_CHECK_EN_PRNDATAUNRECEIVECANCEL_OFF:
                    objOptionParam.SetPrnDataUnreceiveCancel(MfCancel.MF_CANCEL_DISABLE);
			        break;
		        default:
			        break;
	        }
        }

        private void ConvertScanParamToConfigData(ref CScanParam objScanParam, ref StructByStep objConfigData)
        {
	        if( objScanParam == null || objConfigData == null )
	        {
		        return;
	        }

	        switch( objScanParam.GetScanMedia() )
	        {
		        case ScanUnit.EPS_BI_SCN_UNIT_CHECKPAPER:
			        objConfigData.nRadio_ScanningMedia = ConstComVal.VAL_CONFIGDLG_RADIO_SM_CHECKPAPER;
			        break;
                case ScanUnit.EPS_BI_SCN_UNIT_CARD:
			        objConfigData.nRadio_ScanningMedia = ConstComVal.VAL_CONFIGDLG_RADIO_SM_CARD;
			        break;
		        default:
			        break;
	        }

	        switch( objScanParam.GetImageChannel() )	// Scan Gradation
	        {
		        case ImageTypeOption.EPS_BI_SCN_OPTION_COLOR:
                case ImageTypeOption.EPS_BI_SCN_OPTION_GRAYSCALE:
                case ImageTypeOption.EPS_BI_SCN_OPTION_DROPOUT_RED:
                case ImageTypeOption.EPS_BI_SCN_OPTION_DROPOUT_GREEN:
                case ImageTypeOption.EPS_BI_SCN_OPTION_DROPOUT_BLUE:
			        objConfigData.nRadio_ImageChannel = ConstComVal.VAL_CONFIGDLG_RADIO_IC_RGB;
			        break;
                case ImageTypeOption.EPS_BI_SCN_OPTION_IR:
			        objConfigData.nRadio_ImageChannel = ConstComVal.VAL_CONFIGDLG_RADIO_IC_IR;
			        break;
                case ImageTypeOption.EPS_BI_SCN_OPTION_COLOR_IR:
                case ImageTypeOption.EPS_BI_SCN_OPTION_GRAYSCALE_IR:
                case ImageTypeOption.EPS_BI_SCN_OPTION_DROPOUT_RED_IR:
                case ImageTypeOption.EPS_BI_SCN_OPTION_DROPOUT_GREEN_IR:
                case ImageTypeOption.EPS_BI_SCN_OPTION_DROPOUT_BLUE_IR:
			        objConfigData.nRadio_ImageChannel = ConstComVal.VAL_CONFIGDLG_RADIO_IC_RGBIR;
			        break;
		        default:
			        break;
	        }

	        switch( objScanParam.GetRGBColorDepth() )	// RGB Color Depth
	        {
                case com.epson.bank.driver.ColorDepth.EPS_BI_SCN_24BIT:
			        objConfigData.nRadio_RGBColorDepth = ConstComVal.VAL_CONFIGDLG_RADIO_RGBCD_COLOR;
			        break;
                case com.epson.bank.driver.ColorDepth.EPS_BI_SCN_8BIT:
			        objConfigData.nRadio_RGBColorDepth = ConstComVal.VAL_CONFIGDLG_RADIO_RGBCD_GRAY;
			        break;
                case com.epson.bank.driver.ColorDepth.EPS_BI_SCN_1BIT:
			        objConfigData.nRadio_RGBColorDepth = ConstComVal.VAL_CONFIGDLG_RADIO_RGBCD_BW;
			        break;
		        default:
			        break;
	        }

	        switch( objScanParam.GetIRColorDepth() )	// IR Color Depth
	        {
                case com.epson.bank.driver.ColorDepth.EPS_BI_SCN_8BIT:
			        objConfigData.nRadio_IRColorDepth = ConstComVal.VAL_CONFIGDLG_RADIO_IRCD_GRAY;
			        break;
                case com.epson.bank.driver.ColorDepth.EPS_BI_SCN_1BIT:
			        objConfigData.nRadio_IRColorDepth = ConstComVal.VAL_CONFIGDLG_RADIO_IRCD_BW;
			        break;
		        default:
			        break;
	        }

	        switch( objScanParam.GetResolution() )	// Resoluton
	        {
		        case MfScanDpi.MF_SCAN_DPI_600:
			        objConfigData.nCombo_Resolution = ConstComVal.VAL_CONFIGDLG_COMBO_RE_600600;
			        break;
                case MfScanDpi.MF_SCAN_DPI_300:
			        objConfigData.nCombo_Resolution = ConstComVal.VAL_CONFIGDLG_COMBO_RE_300300;
			        break;
                case MfScanDpi.MF_SCAN_DPI_240:
			        objConfigData.nCombo_Resolution = ConstComVal.VAL_CONFIGDLG_COMBO_RE_240240;
			        break;
                case MfScanDpi.MF_SCAN_DPI_200:
			        objConfigData.nCombo_Resolution = ConstComVal.VAL_CONFIGDLG_COMBO_RE_200200;
			        break;
                case MfScanDpi.MF_SCAN_DPI_120:
			        objConfigData.nCombo_Resolution = ConstComVal.VAL_CONFIGDLG_COMBO_RE_120120;
			        break;
                case MfScanDpi.MF_SCAN_DPI_100:
			        objConfigData.nCombo_Resolution = ConstComVal.VAL_CONFIGDLG_COMBO_RE_100100;
			        break;
		        default:
			        break;
	        }
        }

        private void ConvertMicrParamToConfigData( ref CMicrParam objMicrParam, ref StructByStep objConfigData )
        {
	        if( objMicrParam == null || objConfigData == null )
	        {
		        return;
	        }

	        switch( objMicrParam.GetFont() )
	        {
		        case MfMicrFont.MF_MICR_FONT_E13B:
			        objConfigData.nRadio_MICRFont = ConstComVal.VAL_CONFIGDLG_RADIO_MF_E13B;
			        break;
                case MfMicrFont.MF_MICR_FONT_CMC7:
			        objConfigData.nRadio_MICRFont = ConstComVal.VAL_CONFIGDLG_RADIO_MF_CMC7;
			        break;
		        default:
			        break;
	        }

	        if( objMicrParam.GetClearSpace() == true )
	        {
		        objConfigData.bCheck_MICR_ClearSpace = ConstComVal.VAL_CONFIGDLG_CHECK_MI_CLEARSPACE_ON;
	        }
	        else
	        {
		        objConfigData.bCheck_MICR_ClearSpace = ConstComVal.VAL_CONFIGDLG_CHECK_MI_CLEARSPACE_OFF;
	        }
        }

        ////////////////
        //// Step 4
        ////////////////
        private void ConvertEndorseParamToConfigData( ref CEndorseParam objEndorseParam, ref StructByStep objConfigData )
        {

	        if( objEndorseParam == null || objConfigData == null )
	        {
		        return;
            }

            switch (objEndorseParam.GetEndorseType())	// Endorse Type
            {
                case PrintingStation.MF_ST_PHYSICAL_ENDORSEMENT:
                    objConfigData.nRadio_PrintType = ConstComVal.VAL_CONFIGDLG_RADIO_PT_PHYSICAL;
                    break;
                case PrintingStation.MF_ST_E_ENDORSEMENT:
                    objConfigData.nRadio_PrintType = ConstComVal.VAL_CONFIGDLG_RADIO_PT_ELECTRONIC;
                    break;
                default:
                    break;
            }

	        if( objEndorseParam.GetPrintText() == true )
	        {
		        objConfigData.bCheck_Endorsement_PrintText = ConstComVal.VAL_CONFIGDLG_CHECK_EN_PRINTTEXT_ON;
	        }
	        else
	        {
		        objConfigData.bCheck_Endorsement_PrintText = ConstComVal.VAL_CONFIGDLG_CHECK_EN_PRINTTEXT_OFF;
	        }

	        if( objEndorseParam.GetPrintImage() == true )
	        {
		        objConfigData.bCheck_Endorsement_PrintImage = ConstComVal.VAL_CONFIGDLG_CHECK_EN_PRINTIMAGE_ON;
	        }
	        else
	        {
		        objConfigData.bCheck_Endorsement_PrintImage = ConstComVal.VAL_CONFIGDLG_CHECK_EN_PRINTIMAGE_OFF;
	        }
        }

        private void ConvertOptionParamToConfigData( ref COptionParam objOptionParam, ref StructByStep objConfigData )
        {

            if ( objOptionParam == null || objConfigData == null )
            {
                return;
            }

            switch (objOptionParam.GetEndorsePrintMode())
	        {
                case MfEndorsePrintMode.MF_ENDORSEPRINT_MODE_HIGHSPEED:
                    objConfigData.nRadio_PrintMode = ConstComVal.VAL_CONFIGDLG_RADIO_PM_HIGHSPEEDMODE;
			        break;
                case MfEndorsePrintMode.MF_ENDORSEPRINT_MODE_DATAWAITING:
                    objConfigData.nRadio_PrintMode = ConstComVal.VAL_CONFIGDLG_RADIO_PM_DATAWAITINGMODE;
			        break;
		        default:
			        break;
	        }

            switch (objOptionParam.GetPrnDataUnreceiveCancel())
	        {
                case MfCancel.MF_CANCEL_DISABLE:
                    objConfigData.bCheck_Endorsement_PrnDataUnreceiveCancel = ConstComVal.VAL_CONFIGDLG_CHECK_EN_PRNDATAUNRECEIVECANCEL_OFF;
			        break;
                case MfCancel.MF_CANCEL_ENABLE:
                    objConfigData.bCheck_Endorsement_PrnDataUnreceiveCancel = ConstComVal.VAL_CONFIGDLG_CHECK_EN_PRNDATAUNRECEIVECANCEL_ON;
			        break;
		        default:
			        break;
	        }
        }

        private void UpdateImageData( ScanSide eFace, ImageType eImage, bool bFourSheetScan )
        {
            if (m_objDriverControl == null)
	        {
		        return;
	        }

	        CImageResult cImageResult = new CImageResult();

	        // Obtain image data
	        cImageResult.SetFace( eFace );
	        cImageResult.SetGradation( eImage );
	        cImageResult.SetFormat( Format.EPS_BI_SCN_BITMAP );
	        ErrorCode errResult = ErrorCode.SUCCESS;

	        errResult = m_objDriverControl.GetScanImage( ref cImageResult );

            CallbackDisplayImageData(errResult, cImageResult, bFourSheetScan);
        }

        private void CallbackDisplayImageData(ErrorCode errResult, CImageResult cImageResult, bool bFourSheetScan)
        {
            if (this.InvokeRequired)
            {
                this.BeginInvoke(CallbackFuncDisplayImageData, new object[] { errResult, cImageResult, bFourSheetScan });
            }
            else
            {
                if (errResult != ErrorCode.SUCCESS)
                {
                    ShowErrorMessage(errResult);
                    return;
                }

                Bitmap cBitmap = new Bitmap(cImageResult.GetBitmapImage());
                if (cBitmap != null)
                {
                    cImageResult.FreeImage();
                }

                if (bFourSheetScan == false)
                {
                    if (cImageResult.GetFace() == ScanSide.MF_SCAN_FACE_FRONT)
                    {
                        m_biImage2Front = new Bitmap(cBitmap);
                    }
                    else
                    {
                        m_biImage2Back = new Bitmap(cBitmap);
                    }
                }
                else
                {
                    if (cImageResult.GetGradation() == ImageType.MF_SCAN_IMAGE_VISIBLE)
                    {
                        if (cImageResult.GetFace() == ScanSide.MF_SCAN_FACE_FRONT)
                        {
                            m_biImage4RGBFront = new Bitmap(cBitmap);
                        }
                        else
                        {
                            m_biImage4RGBBack = new Bitmap(cBitmap);
                        }
                    }
                    else
                    {
                        if (cImageResult.GetFace() == ScanSide.MF_SCAN_FACE_FRONT)
                        {
                            m_biImage4IrFront = new Bitmap(cBitmap);
                        }
                        else
                        {
                            m_biImage4IrBack = new Bitmap(cBitmap);
                        }
                    }
                }

                cBitmap.Dispose();
                this.Refresh();
            }
        }

        private void SaveImageData( ScanSide eFace, ImageType eImage )
        {
            if (m_objDriverControl == null)
	        {
		        return;
	        }

	        CImageResult cImageResult = new CImageResult();

	        // Specify image data format
	        Format eFormat = Format.EPS_BI_SCN_JPEGNORMAL;
	        if( eImage == ImageType.MF_SCAN_IMAGE_VISIBLE &&  m_objConfigData.nRadio_RGBColorDepth == ConstComVal.VAL_CONFIGDLG_RADIO_RGBCD_BW )
	        {
                eFormat = Format.EPS_BI_SCN_TIFF;
	        }
	        if( eImage == ImageType.MF_SCAN_IMAGE_INFRARED &&  m_objConfigData.nRadio_IRColorDepth == ConstComVal.VAL_CONFIGDLG_RADIO_IRCD_BW )
	        {
                eFormat = Format.EPS_BI_SCN_TIFF;
	        }

	        // Obtain image data
            cImageResult.SetFace(eFace);
	        cImageResult.SetGradation( eImage );
	        cImageResult.SetFormat( eFormat );
            ErrorCode errResult = ErrorCode.SUCCESS;

            errResult = m_objDriverControl.GetScanImage(ref cImageResult);

            CallbackSaveImageData(errResult, cImageResult);
        }

        private void CallbackSaveImageData(ErrorCode errResult, CImageResult objImageResult)
        {
            if (this.InvokeRequired)
            {
                this.BeginInvoke(CallbackFuncSaveImageData, new object[] { errResult, objImageResult });
            }
            else
            {
                if (errResult != ErrorCode.SUCCESS)
                {
                    ShowErrorMessage(errResult);
                    return;
                }

                Stream cImage = objImageResult.GetImageData();
                int nImageSize = objImageResult.GetImageSize();
                int nTransactionNumber = objImageResult.GetTransactionNumber();
                string strExt;

                if (objImageResult.GetFormat() == Format.EPS_BI_SCN_TIFF)
                {
                    strExt = "tif";
                }
                else
                {
                    strExt = "jpg";
                }


                // Create Folder
                StringBuilder sbPath = new StringBuilder();
                string szPath = null;

                sbPath.Append(System.IO.Directory.GetCurrentDirectory());
                sbPath.Append("\\ScanResult");
                szPath = sbPath.ToString();

                try
                {
                    System.IO.Directory.CreateDirectory(szPath);
                }
                catch
                {
                    MessageBox.Show(ConstComStr.MSG_05_000, ConstComStr.CAPTION_05_000, MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }


                // Save image to file
                sbPath.Append("\\" + nTransactionNumber.ToString("000") + ("_Image_")
                                                 + (objImageResult.GetGradation() == ImageType.MF_SCAN_IMAGE_INFRARED ? "IR_" : "")
                                                 + (objImageResult.GetFace() == ScanSide.MF_SCAN_FACE_BACK ? "Back" : "Front") + (".")
                                                 + strExt);
                szPath = sbPath.ToString();

                if (objImageResult.GetImageData() != null)
                {
                    System.IO.FileStream cFile;
                    try
                    {
                        cFile = new System.IO.FileStream(szPath, System.IO.FileMode.Create);
                    }
                    catch
                    {
                        MessageBox.Show(ConstComStr.MSG_05_001, ConstComStr.CAPTION_05_000, MessageBoxButtons.OK, MessageBoxIcon.Error);
                        return;
                    }

                    byte[] byBuffer = new byte[objImageResult.GetImageSize()];

                    // Save in file
                    try
                    {
                        objImageResult.GetImageData().Read(byBuffer, 0, byBuffer.Length);
                        cFile.Write(byBuffer, 0, byBuffer.Length);

                        objImageResult.GetImageData().Close();
                    }
                    catch
                    {
                        MessageBox.Show(ConstComStr.MSG_05_001, ConstComStr.CAPTION_05_001, MessageBoxButtons.OK, MessageBoxIcon.Error);
                        return;
                    }
                    finally
                    {
                        cFile.Close();
                    }
                }
            }
        }
        
        private void ClearImage()
        {
            if (stImage2Front.Image != null)
            {
                stImage2Front.Image.Dispose();
                stImage2Front.Image = null;
                m_biImage2Front     = null;
            }
            if (stImage2Back.Image != null)
            {
                stImage2Back.Image.Dispose();
                stImage2Back.Image  = null;
                m_biImage2Back      = null;
            }
            if (stImage4RGBFront.Image != null)
            {
                stImage4RGBFront.Image.Dispose();
                stImage4RGBFront.Image  = null;
                m_biImage4RGBFront      = null;
            }
            if (stImage4RGBBack.Image != null)
            {
                stImage4RGBBack.Image.Dispose();
                stImage4RGBBack.Image   = null;
                m_biImage4RGBBack       = null;
            }
            if (stImage4IRBack.Image != null)
            {
                stImage4IRBack.Image.Dispose();
                stImage4IRBack.Image    = null;
                m_biImage4IrBack        = null;
            }
            if (stImage4IRFront.Image != null)
            {
                stImage4IRFront.Image.Dispose();
                stImage4IRFront.Image   = null;
                m_biImage4IrFront       = null;
            }
        }

        private void DrawImage(PictureBox stImage, Bitmap biImage)
        {
            int nWidth = 0;
            int nHeight = 0;

            if ((biImage.Width * stImage.Height) < (stImage.Width * biImage.Height))
            {
                nWidth = stImage.Height * biImage.Width / biImage.Height;
                stImage.Image = new Bitmap(biImage, nWidth, stImage.Height);
            }
            else
            {
                nHeight = stImage.Width * biImage.Height / biImage.Width;
                stImage.Image = new Bitmap(biImage, stImage.Width, nHeight);
            }
        }

        private void CallbackSaveMicrText(ErrorCode errResult, CMicrResult cMicrResult)
        {
            if (this.InvokeRequired)
            {
                this.BeginInvoke(CallbackFuncSaveMicrText, new object[] { errResult, cMicrResult });
            }
            else
            {
                // Obtain MICR data
                int nTransactionNumber = cMicrResult.GetTransactionNumber();

                if (errResult != ErrorCode.SUCCESS)
                {
                    ShowErrorMessage(errResult);
                    return;
                }

                // Create Folder
                StringBuilder sbPath = new StringBuilder();
                string szPath = null;

                sbPath.Append(System.IO.Directory.GetCurrentDirectory());
                sbPath.Append("\\ScanResult");
                szPath = sbPath.ToString();
                try
                {
                    System.IO.Directory.CreateDirectory(szPath);
                }
                catch
                {
                    MessageBox.Show(ConstComStr.MSG_05_000, ConstComStr.CAPTION_05_000, MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }

                // Save data to file
                sbPath.Append("\\" + nTransactionNumber.ToString("000") + ".Micr.txt");
                szPath = sbPath.ToString();

                System.IO.StreamWriter swFile;
                try
                {
                    swFile = new System.IO.StreamWriter(szPath);
                }
                catch
                {
                    MessageBox.Show(ConstComStr.MSG_05_001, ConstComStr.CAPTION_05_001, MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }

                try
                {
                    // Transaction number
                    swFile.WriteLine("# General Transaction Information:");
                    swFile.WriteLine("TransactionNumber = " + nTransactionNumber.ToString());

                    // MICR reading status
                    swFile.WriteLine("");
                    swFile.WriteLine("# Magnetic Character Results:");
                    swFile.WriteLine("Status = " + cMicrResult.GetStatus().ToString());

                    // Detail MICR reading status
                    swFile.WriteLine("Detail = " + cMicrResult.GetDetail().ToString());

                    // Return value from MICR recognition
                    swFile.WriteLine("MicrResult = " + GetErrorString(cMicrResult.GetResult()));

                    // Obtained MICR strings
                    swFile.WriteLine("MicrString = " + (cMicrResult.GetMicrStr() != null ? cMicrResult.GetMicrStr() : ""));

                    //MicrAccountNumber
                    swFile.WriteLine("MicrAccountNumber = " + (cMicrResult.GetAccountNumber() != null ? cMicrResult.GetAccountNumber() : ""));

                    //MicrAmount
                    swFile.WriteLine("MicrAmount        = " + (cMicrResult.GetAmount() != null ? cMicrResult.GetAmount() : ""));

                    //MicrBankNumber
                    swFile.WriteLine("MicrBankNumber    = " + (cMicrResult.GetBankNumber() != null ? cMicrResult.GetBankNumber() : ""));

                    //MicrSerialNumber
                    swFile.WriteLine("MicrSerialNumber  = " + (cMicrResult.GetSerialNumber() != null ? cMicrResult.GetSerialNumber() : ""));

                    //MicrEPC
                    swFile.WriteLine("MicrEPC           = " + (cMicrResult.GetEPC() != null ? cMicrResult.GetEPC() : ""));

                    // MicrTransitNumber
                    swFile.WriteLine("MicrTransitNumber = " + (cMicrResult.GetTransitNumber() != null ? cMicrResult.GetTransitNumber() : ""));

                    // MicrCheckType
                    swFile.WriteLine("MicrCheckType     = " + cMicrResult.GetCheckType().ToString());

                    // MicrCountryCode
                    swFile.WriteLine("MicrCountryCode     = " + cMicrResult.GetCountryCode().ToString());

                    // MicrOnUSField
                    swFile.WriteLine("MicrOnUSField = " + (cMicrResult.GetOnUSField() != null ? cMicrResult.GetOnUSField() : ""));

                    // MicrAuxiliatyOnUSField
                    swFile.WriteLine("MicrAuxiliatyOnUSField = " + (cMicrResult.GetAuxiliatyOnUSField() != null ? cMicrResult.GetAuxiliatyOnUSField() : ""));

                }
                catch
                {
                    MessageBox.Show(ConstComStr.MSG_05_001, ConstComStr.CAPTION_05_001, MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
                finally
                {
                    swFile.Close();
                }
            }
        }
    }
}
