#include "stdafx.h"
#include "Image_1.h"

CImage_1::CImage_1(void)
{
	m_pCProperty = NULL;
}

CImage_1::~CImage_1(void)
{
}

int CImage_1::Init(CProperty_1* pCCProperty)
{
	int nResult = SUCCESS;

	m_pCProperty = pCCProperty;

	GetAPIProcAddress();

	return nResult;
}

// Specify image data format
int CImage_1::SetImageSetting(BYTE byFace, BYTE byGradation, BYTE byFormat)
{
	int nResult = SUCCESS;
	BYTE byColorDepth = EPS_BI_SCN_24BIT;
	BYTE byColor = EPS_BI_SCN_COLOR;
	BYTE byExOption = EPS_BI_SCN_MANUAL;

	nResult = BiSCNSelectScanFace(m_pCProperty->GetHandle(), byFace);
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	nResult = BiSCNSelectScanImage(m_pCProperty->GetHandle(), byGradation);
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	if(byGradation == MF_SCAN_IMAGE_VISIBLE)
	{
		byColorDepth = m_pCProperty->GetRGBColorDepth();
	}
	else
	{
		byColorDepth = m_pCProperty->GetIRColorDepth();
	}

	if(byColorDepth == EPS_BI_SCN_24BIT)
	{
		byColor = EPS_BI_SCN_COLOR;
		byExOption = EPS_BI_SCN_MANUAL;
	}
	else
	{
		byColor = EPS_BI_SCN_MONOCHROME;
		byExOption = EPS_BI_SCN_SHARP;
	}

	nResult = BiSCNSetImageQuality(m_pCProperty->GetHandle(), byColorDepth, 0, byColor, byExOption);
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	nResult = BiSCNSetImageFormat(m_pCProperty->GetHandle(), byFormat);

	return nResult;
}

// Get check image
int CImage_1::GetScanImage(CImageResult* pCCImageResult)
{
	int nResult = SUCCESS;
	MF_SCAN* pMfScan = NULL;

	if( pCCImageResult == NULL )
	{
		return -1;
	}

	nResult = SetImageSetting(pCCImageResult->GetFace(), pCCImageResult->GetGradation(), pCCImageResult->GetFormat());
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	if(pCCImageResult->GetFace() == MF_SCAN_FACE_FRONT)
	{
		pMfScan = m_pCProperty->GetMfScanFront();
	}
	else
	{
		pMfScan = m_pCProperty->GetMfScanBack();
	}

	nResult = BiGetScanImage(m_pCProperty->GetHandle(), m_pCProperty->GetTransactionNumber(), pMfScan);
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	pCCImageResult->SetTransactionNumber(m_pCProperty->GetTransactionNumber());
	pCCImageResult->SetImageSize(pMfScan->dwScanSize);
	pCCImageResult->SetImage(pMfScan->lpbScanData);

	// Obtained image data is not stored in this class
	pMfScan->dwScanSize = 0;
	pMfScan->lpbScanData = NULL;

	return nResult;
}

// Get cashier's check image
int CImage_1::GetCashiersCheckImage(CImageResult* pCCImageResult)
{
	if( pCCImageResult == NULL )
	{
		return -1;
	}

	int nResult = SUCCESS;
	MF_SCAN* pMfScan = NULL;

	nResult = BiSCNSelectScanFace(m_pCProperty->GetHandle(), pCCImageResult->GetFace());
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	nResult = BiSCNSelectScanImage(m_pCProperty->GetHandle(), MF_SCAN_IMAGE_VISIBLE);
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	nResult = BiSCNSetImageQuality(m_pCProperty->GetHandle(), EPS_BI_SCN_24BIT, 0, EPS_BI_SCN_COLOR, EPS_BI_SCN_MANUAL);
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	nResult = BiSCNSetImageFormat(m_pCProperty->GetHandle(), EPS_BI_SCN_BITMAP);
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	if(pCCImageResult->GetFace() == MF_SCAN_FACE_FRONT)
	{
		pMfScan = m_pCProperty->GetMfScanFront();
	}
	else
	{
		pMfScan = m_pCProperty->GetMfScanBack();
	}


	nResult = BiGetScanImage(m_pCProperty->GetHandle(), m_pCProperty->GetTransactionNumber(), pMfScan);
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	pCCImageResult->SetTransactionNumber(m_pCProperty->GetTransactionNumber());
	pCCImageResult->SetImageSize(pMfScan->dwScanSize);
	pCCImageResult->SetImage(pMfScan->lpbScanData);

	// Obtained image is not stored in this class
	pMfScan->dwScanSize = 0;
	pMfScan->lpbScanData = NULL;

	return nResult;
}

// Get entry point of API
void CImage_1::GetAPIProcAddress(void)
{
	HINSTANCE hLibrary = m_pCProperty->GetLibrary();

	BiSCNSelectScanFace				= (DLL_BiSCNSelectScanFace)				GetProcAddress(hLibrary, "BiSCNSelectScanFace");
	BiSCNSetImageQuality			= (DLL_BiSCNSetImageQuality)			GetProcAddress(hLibrary, "BiSCNSetImageQuality");
	BiSCNSetImageFormat				= (DLL_BiSCNSetImageFormat)				GetProcAddress(hLibrary, "BiSCNSetImageFormat");
	BiGetScanImage					= (DLL_BiGetScanImage)					GetProcAddress(hLibrary, "BiGetScanImage");
	BiSCNSelectScanImage			= (DLL_BiSCNSelectScanImage)			GetProcAddress(hLibrary, "BiSCNSelectScanImage");
}
