#include "stdafx.h"
#include "App.h"

CApp*	g_pCCApp;

CApp::CApp(void)
{
	m_pFuncMicrCB				= NULL;
	m_pFuncImageCB				= NULL;
	m_pFuncProcessErrorCB		= NULL;
	g_pCCApp					= this;
	m_byExecType				= EXEC_SCAN_CHECK;
}

CApp::~CApp(void)
{
	ExitDevice();
	g_pCCApp = NULL;
}

// Initialize
int CApp::InitDevice(void)
{
	int nResult = SUCCESS;

	if(m_CCProperty.GetLibrary() == NULL)
	{
		// Load the TM-S9000 driver
		HINSTANCE hLibrary = ::LoadLibrary(_T("EpsStmApiWrapper.dll"));
		if(hLibrary == NULL)
		{
			return ERR_LOAD_MODULE;
		}
		m_CCProperty.SetLibrary(hLibrary);
	}

	GetAPIProcAddress();

	if(m_CCProperty.GetHandle() <= 0)
	{
		// Connect to the device
		int nHandle = BiOpenMonPrinter(TYPE_PRINTER, "TM-S9000U,TM-S2000U");
		if(nHandle <= 0)
		{
			return nHandle;
		}

		m_CCProperty.SetHandle(nHandle);
	}

	m_CCScan.Init(&m_CCProperty);
	m_CCMicr.Init(&m_CCProperty);
	m_CCOption.Init(&m_CCProperty);
	// Step 2
	m_CCImage.Init(&m_CCProperty);
	// Step 4
	nResult = m_CCEndorse.Init(&m_CCProperty);
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	// Register callback function for notification of the reading status
	nResult = BiSCNMICRSetStatusBackFunction(m_CCProperty.GetHandle(), cbScanStatus);
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	// Register callback function for notification of the endorsement data acceptance
	nResult = BiStartEndorsementSetStatusBackFunction(m_CCProperty.GetHandle(), cbStartEndorsement);
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	// Register callback function for notification of the endorsement data close
	nResult = BiEndEndorsementSetStatusBackFunction(m_CCProperty.GetHandle(), cbEndEndorsement);
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	return nResult;
}

// Post process
int CApp::ExitDevice(void)
{
	int nResult = SUCCESS;

	if(m_CCProperty.GetHandle() > 0)
	{
		// Cancel the reading status callback
		nResult = BiSCNMICRCancelStatusBack(m_CCProperty.GetHandle());
		if(nResult != SUCCESS)
		{
			return nResult;
		}

		// Step 4
		// Cancel the endorsement data acceptance callback
		nResult = BiStartEndorsementCancelStatusBack(m_CCProperty.GetHandle());
		if(nResult != SUCCESS)
		{
			return nResult;
		}

		// Cancel the endorsement data close callback
		nResult = BiEndEndorsementCancelStatusBack(m_CCProperty.GetHandle());
		if(nResult != SUCCESS)
		{
			return nResult;
		}

		// Disconnect the device
		nResult = BiCloseMonPrinter(m_CCProperty.GetHandle());
		if(nResult != SUCCESS)
		{
			return nResult;
		}
		m_CCProperty.SetHandle(0);
	}

	if(m_CCProperty.GetLibrary() != NULL)
	{
		// Unload the driver
		::FreeLibrary(m_CCProperty.GetLibrary());
		m_CCProperty.SetLibrary(NULL);
	}

	return nResult;
}

// Register callback function for notification of the MICR data
void CApp::SetMicrCallback(MICR_IMAGE_BACK_FUNC pMicrCB)
{
	m_pFuncMicrCB = pMicrCB;
}

// Register callback function for notification of the image data
void CApp::SetImageCallback(MICR_IMAGE_BACK_FUNC pImageCB)
{
	m_pFuncImageCB = pImageCB;
}

// Register callback function for notification of error during image acquisition
void CApp::SetProcessErrorCallback(PROCESS_ERROR_BACK_FUNC pProcessErrorCB)
{
	m_pFuncProcessErrorCB = pProcessErrorCB;
}

// Set MICR settings
void CApp::SetMicrParam(CMicrParam* pCCMicrParam)
{
	if(pCCMicrParam != NULL)
	{
		m_CCProperty.SetFont(pCCMicrParam->GetFont());
		m_CCProperty.SetParsing(pCCMicrParam->GetParsing());
		m_CCProperty.SetClearSpace(pCCMicrParam->GetClearSpace());
	}
}

// Get MICR settings
void CApp::GetMicrParam(CMicrParam* pCCMicrParam)
{
	if(pCCMicrParam != NULL)
	{
		pCCMicrParam->SetFont(m_CCProperty.GetFont());
		pCCMicrParam->SetParsing(m_CCProperty.GetParsing());
		pCCMicrParam->SetClearSpace(m_CCProperty.GetClearSpace());
	}
}

// Set scanning parameters
void CApp::SetScanParam(CScanParam* pCCScanParam)
{
	if(pCCScanParam != NULL)
	{
		m_CCProperty.SetScanMedia(pCCScanParam->GetScanMedia());
		m_CCProperty.SetImageChannel(pCCScanParam->GetImageChannel());
		m_CCProperty.SetRGBColorDepth(pCCScanParam->GetRGBColorDepth());
		m_CCProperty.SetIRColorDepth(pCCScanParam->GetIRColorDepth());
		m_CCProperty.SetResolution(pCCScanParam->GetResolution());
	}
}

// Get scanning parameters
void CApp::GetScanParam(CScanParam* pCCScanParam)
{
	if(pCCScanParam != NULL)
	{
		pCCScanParam->SetScanMedia(m_CCProperty.GetScanMedia());
		pCCScanParam->SetImageChannel(m_CCProperty.GetImageChannel());
		pCCScanParam->SetRGBColorDepth(m_CCProperty.GetRGBColorDepth());
		pCCScanParam->SetIRColorDepth(m_CCProperty.GetIRColorDepth());
		pCCScanParam->SetResolution(m_CCProperty.GetResolution());
	}
}

// Check scanning process
int CApp::ScanCheck(void)
{
	int nResult = SUCCESS;

	m_byExecType = EXEC_SCAN_CHECK;

	// Set MF_BASE01 structure
	nResult = m_CCScan.SetBaseSetting();
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	// Set scanning parameters for check scanning
	nResult = m_CCScan.SetScanSetting();
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	// Set MICR parameters
	nResult = m_CCMicr.SetMicrSetting();
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	// Set MF_PROCESS01 structure
	nResult = m_CCOption.SetProcessSetting();
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	// Set endorsement type
	nResult = m_CCEndorse.SetEndorseStation();
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	// Start check scanning
	nResult = m_CCScan.ScanFunction();
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	return nResult;
}

// Card scanning process
int CApp::ScanCard(void)
{
	int nResult = SUCCESS;

	m_byExecType = EXEC_SCAN_CARD;

	// Clear MF_MICR strucuture
	nResult = m_CCMicr.ClearMicrSetting();
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	// Set MF_BASE01 strucuture
	nResult = m_CCScan.SetBaseSetting();
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	// Set card scanning conditions
	nResult = m_CCScan.SetScanSetting();
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	// Start card scanning
	nResult = m_CCScan.ScanFunction();
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	return nResult;
}

// Get MICR data
int CApp::GetMicr(CMicrResult* pCCMicrResult)
{
	return m_CCMicr.GetMicr(pCCMicrResult);
}

// Get scanned image
int CApp::GetScanImage(CImageResult* pCCImageResult)
{
	return m_CCImage.GetScanImage(pCCImageResult);
}

// Error recovery
int CApp::CancelError(void)
{
	return BiCancelError(m_CCProperty.GetHandle());
}

// Get device status
int CApp::GetDeviceStatus(LPDWORD lpStatus)
{
	return BiGetStatus(m_CCProperty.GetHandle(), lpStatus);
}

// Cancel scanning
int CApp::CancelScan(void)
{
	return m_CCScan.CancelScan();
}

// MICR head cleaning
int CApp::CleaningMicr(void)
{
	return m_CCMicr.CleaningMicr();
}

// Get functions of the API
void CApp::GetAPIProcAddress(void)
{
	HINSTANCE hLibrary = m_CCProperty.GetLibrary();

	BiOpenMonPrinter						= (DLL_BiOpenMonPrinter)						GetProcAddress(hLibrary, "BiOpenMonPrinter");
	BiCloseMonPrinter						= (DLL_BiCloseMonPrinter)						GetProcAddress(hLibrary, "BiCloseMonPrinter");
	BiSCNMICRSetStatusBackFunction			= (DLL_BiSCNMICRSetStatusBackFunction)			GetProcAddress(hLibrary, "BiSCNMICRSetStatusBackFunction");
	BiSCNMICRCancelStatusBack				= (DLL_BiSCNMICRCancelStatusBack)				GetProcAddress(hLibrary, "BiSCNMICRCancelStatusBack");
	BiCancelError							= (DLL_BiCancelError)							GetProcAddress(hLibrary, "BiCancelError");
	BiGetStatus								= (DLL_BiGetStatus)								GetProcAddress(hLibrary, "BiGetStatus");
	// Step 4
	BiStartEndorsementSetStatusBackFunction = (DLL_BiStartEndorsementSetStatusBackFunction)	GetProcAddress(hLibrary, "BiStartEndorsementSetStatusBackFunction");
	BiStartEndorsementCancelStatusBack		= (DLL_BiStartEndorsementCancelStatusBack)		GetProcAddress(hLibrary, "BiStartEndorsementCancelStatusBack");
	BiEndEndorsementSetStatusBackFunction	= (DLL_BiEndEndorsementSetStatusBackFunction)	GetProcAddress(hLibrary, "BiEndEndorsementSetStatusBackFunction");
	BiEndEndorsementCancelStatusBack		= (DLL_BiEndEndorsementCancelStatusBack)		GetProcAddress(hLibrary, "BiEndEndorsementCancelStatusBack");
}

// Callback function for notifying scanning status
int CApp::cbScanStatus(DWORD dwTransactionNumber, WORD wMainStatus, WORD wSubStatus, LPSTR pPortName)
{
	return g_pCCApp->ScanStatus(dwTransactionNumber, wMainStatus, wSubStatus, pPortName);
}

int CApp::ScanStatus(DWORD dwTransactionNumber, WORD wMainStatus, WORD wSubStatus, LPSTR pPortName)
{
	if(wSubStatus != SUCCESS)
	{
		m_pFuncProcessErrorCB( static_cast<short>(wSubStatus) );
	}

	// Step 2
	if(wMainStatus == MF_DATARECEIVE_DONE)
	{
		m_CCProperty.SetTransactionNumber(dwTransactionNumber);

		// Step 4
		if(m_CCProperty.GetEndorseType() == MF_ST_E_ENDORSEMENT)
		{
			if(m_CCProperty.GetPrintText() || m_CCProperty.GetPrintImage())
			{
				// Notify callback for endorsement
				m_pFuncElectronicEndorseCB();
			}
		}

		// Step 3
		if( m_byExecType == EXEC_SCAN_CHECK )
		{
			// Notify callback for MICR data
			m_pFuncMicrCB();
		}
		// Notify callback for image data
		m_pFuncImageCB();
	}

	return 0;
}

////////////////
//// Step 4 ////
////////////////

// Register callback for Physical Endorsement
void CApp::SetPhysicalEndorseCallback(PHYSICAL_ENDORSE_BACK_FUNC pPhysicalEndorseCB)
{
	m_pFuncPhysicalEndorseCB = pPhysicalEndorseCB;
}

// Register callback for Electronic Endorsement
void CApp::SetElectronicEndorseCallback(ELECTRONIC_ENDORSE_BACK_FUNC pElectronicEndorseCB)
{
	m_pFuncElectronicEndorseCB = pElectronicEndorseCB;
}

// Reading endorsement data file
int CApp::LoadEndorseFile(void)
{
	int nResult = SUCCESS;

	nResult = m_CCEndorse.LoadEndorseFile();

	return nResult;
}

// Set endorsement data
void CApp::SetEndorseParam(CEndorseParam* pCCEndorseParam)
{
	if(pCCEndorseParam != NULL)
	{
		m_CCProperty.SetEndorseType(pCCEndorseParam->GetEndorseType());
		m_CCProperty.SetPrintText(pCCEndorseParam->GetPrintText());
		m_CCProperty.SetPrintImage(pCCEndorseParam->GetPrintImage());
	}
}

// Get endorsement data
void CApp::GetEndorseParam(CEndorseParam* pCCEndorseParam)
{
	if(pCCEndorseParam != NULL)
	{
		pCCEndorseParam->SetEndorseType(m_CCProperty.GetEndorseType());
		pCCEndorseParam->SetPrintText(m_CCProperty.GetPrintText());
		pCCEndorseParam->SetPrintImage(m_CCProperty.GetPrintImage());
	}
}

// Set processing mode options
void CApp::SetOptionParam(COptionParam* pCCOptionParam)
{
	if(pCCOptionParam != NULL)
	{
		m_CCProperty.SetPrnDataUnreceiveCancel(pCCOptionParam->GetPrnDataUnreceiveCancel());
		m_CCProperty.SetEndorsePrintMode(pCCOptionParam->GetEndorsePrintMode());
	}
}

// Get processing mode options
void CApp::GetOptionParam(COptionParam* pCCOptionParam)
{
	if(pCCOptionParam != NULL)
	{
		pCCOptionParam->SetPrnDataUnreceiveCancel(m_CCProperty.GetPrnDataUnreceiveCancel());
		pCCOptionParam->SetEndorsePrintMode(m_CCProperty.GetEndorsePrintMode());
	}
}

// Set Physical Endorsement data
int CApp::PhysicalEndorseFunction(unsigned long ulTransactionNumber)
{
	int nResult = SUCCESS;

	nResult = m_CCEndorse.PhysicalEndorseFunction(ulTransactionNumber);

	return nResult;
}

// Set Electronic Endorsement data
int CApp::ElectronicEndorseFunction(void)
{
	int nResult = SUCCESS;

	nResult = m_CCEndorse.ElectronicEndorseFunction();

	return nResult;
}

// Callback function for notifying endorsement data acceptance
int CApp::cbStartEndorsement(unsigned long ulTransactionNumber, LPSTR lpcPortName)
{
	return g_pCCApp->StartEndorsement(ulTransactionNumber, lpcPortName);
}

int CApp::StartEndorsement(unsigned long ulTransactionNumber, LPSTR lpcPortName)
{
	if(m_CCProperty.GetEndorseType() == MF_ST_PHYSICAL_ENDORSEMENT)
	{
		if(m_CCProperty.GetPrintText() || m_CCProperty.GetPrintImage())
		{
			// notifying endorsement data acceptance
			m_pFuncPhysicalEndorseCB(ulTransactionNumber);
		}
	}

	return 0;
}

// Callback function for notifying endorsement data close
int CApp::cbEndEndorsement(unsigned long ulTransactionNumber, LPSTR lpcPortName)
{
	return g_pCCApp->EndEndorsement(ulTransactionNumber, lpcPortName);
}

int CApp::EndEndorsement(unsigned long ulTransactionNumber, LPSTR lpcPortName)
{
	/* After this callback is executed, all endorsement data is discarded */
	return 0;
}
