#include "stdafx.h"
#include "App.h"

CApp*	g_pCCApp;

CApp::CApp(void)
{
	m_pFuncMicrCB				= NULL;
	m_pFuncImageCB				= NULL;
	m_pFuncProcessErrorCB		= NULL;
	g_pCCApp					= this;
	m_byExecType				= EXEC_SCAN_CHECK;
}

CApp::~CApp(void)
{
	ExitDevice();
	g_pCCApp = NULL;
}

// Initialize
int CApp::InitDevice(void)
{
	int nResult = SUCCESS;

	if(m_CCProperty.GetLibrary() == NULL)
	{
		// Load the TM-S9000 driver
		HINSTANCE hLibrary = ::LoadLibrary(_T("EpsStmApiWrapper.dll"));
		if(hLibrary == NULL)
		{
			return ERR_LOAD_MODULE;
		}
		m_CCProperty.SetLibrary(hLibrary);
	}

	GetAPIProcAddress();

	if(m_CCProperty.GetHandle() <= 0)
	{
		// Connect to the device
		int nHandle = BiOpenMonPrinter(TYPE_PRINTER, "TM-S9000U,TM-S2000U");
		if(nHandle <= 0)
		{
			return nHandle;
		}

		m_CCProperty.SetHandle(nHandle);
	}

	m_CCScan.Init(&m_CCProperty);
	m_CCMicr.Init(&m_CCProperty);
	m_CCOption.Init(&m_CCProperty);
	// Step 2
	m_CCImage.Init(&m_CCProperty);
	// Step 5
	m_CCPrint.Init(&m_CCProperty);

	// Register callback function for notification of the reading status
	nResult = BiSCNMICRSetStatusBackFunction(m_CCProperty.GetHandle(), cbScanStatus);
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	// Register callback function for notification of the endorsement data acceptance
	nResult = BiStartEndorsementSetStatusBackFunction(m_CCProperty.GetHandle(), cbStartEndorsement);
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	// Register callback function for notification of the endorsement data close
	nResult = BiEndEndorsementSetStatusBackFunction(m_CCProperty.GetHandle(), cbEndEndorsement);
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	return nResult;
}

// Post process
int CApp::ExitDevice(void)
{
	int nResult = SUCCESS;

	if(m_CCProperty.GetHandle() > 0)
	{
		// Cancel the reading status callback
		nResult = BiSCNMICRCancelStatusBack(m_CCProperty.GetHandle());
		if(nResult != SUCCESS)
		{
			return nResult;
		}

		// Step 5
		// Cancel the endorsement data acceptance callback
		nResult = BiStartEndorsementCancelStatusBack(m_CCProperty.GetHandle());
		if(nResult != SUCCESS)
		{
			return nResult;
		}

		// Cancel the endorsement data close callback
		nResult = BiEndEndorsementCancelStatusBack(m_CCProperty.GetHandle());
		if(nResult != SUCCESS)
		{
			return nResult;
		}

		// Disconnect the device
		nResult = BiCloseMonPrinter(m_CCProperty.GetHandle());
		if(nResult != SUCCESS)
		{
			return nResult;
		}
		m_CCProperty.SetHandle(0);
	}

	if(m_CCProperty.GetLibrary() != NULL)
	{
		// Unload the driver
		::FreeLibrary(m_CCProperty.GetLibrary());
		m_CCProperty.SetLibrary(NULL);
	}

	return nResult;
}

// Register callback function for notification of the MICR data
void CApp::SetMicrCallback(MICR_IMAGE_BACK_FUNC pMicrCB)
{
	m_pFuncMicrCB = pMicrCB;
}

// Register callback function for notification of the image data
void CApp::SetImageCallback(MICR_IMAGE_BACK_FUNC pImageCB)
{
	m_pFuncImageCB = pImageCB;
}

// Register callback function for notification of error during image acquisition
void CApp::SetProcessErrorCallback(PROCESS_ERROR_BACK_FUNC pProcessErrorCB)
{
	m_pFuncProcessErrorCB = pProcessErrorCB;
}

// Set MICR settings
void CApp::SetMicrParam(CMicrParam* pCCMicrParam)
{
	if(pCCMicrParam != NULL)
	{
		m_CCProperty.SetFont(pCCMicrParam->GetFont());
		m_CCProperty.SetParsing(pCCMicrParam->GetParsing());
		m_CCProperty.SetClearSpace(pCCMicrParam->GetClearSpace());
	}
}

// Get MICR settings
void CApp::GetMicrParam(CMicrParam* pCCMicrParam)
{
	if(pCCMicrParam != NULL)
	{
		pCCMicrParam->SetFont(m_CCProperty.GetFont());
		pCCMicrParam->SetParsing(m_CCProperty.GetParsing());
		pCCMicrParam->SetClearSpace(m_CCProperty.GetClearSpace());
	}
}

// Set scanning parameters
void CApp::SetScanParam(CScanParam* pCCScanParam)
{
	if(pCCScanParam != NULL)
	{
		m_CCProperty.SetScanMedia(pCCScanParam->GetScanMedia());
		m_CCProperty.SetImageChannel(pCCScanParam->GetImageChannel());
		m_CCProperty.SetRGBColorDepth(pCCScanParam->GetRGBColorDepth());
		m_CCProperty.SetIRColorDepth(pCCScanParam->GetIRColorDepth());
		m_CCProperty.SetResolution(pCCScanParam->GetResolution());
	}
}

// Get scanning parameters
void CApp::GetScanParam(CScanParam* pCCScanParam)
{
	if(pCCScanParam != NULL)
	{
		pCCScanParam->SetScanMedia(m_CCProperty.GetScanMedia());
		pCCScanParam->SetImageChannel(m_CCProperty.GetImageChannel());
		pCCScanParam->SetRGBColorDepth(m_CCProperty.GetRGBColorDepth());
		pCCScanParam->SetIRColorDepth(m_CCProperty.GetIRColorDepth());
		pCCScanParam->SetResolution(m_CCProperty.GetResolution());
	}
}

// Set printing parameters
void CApp::SetPrintParam(CPrintParam* pCCPrintParam)
{
	if(pCCPrintParam != NULL)
	{
		m_CCProperty.SetAfterScanning(pCCPrintParam->GetAfterScanning());
		m_CCProperty.SetPrintingNumber(pCCPrintParam->GetPrintingNumber());
	}
}

// Get printing parameters
void CApp::GetPrintParam(CPrintParam* pCCPrintParam)
{
	if(pCCPrintParam != NULL)
	{
		pCCPrintParam->SetAfterScanning(m_CCProperty.GetAfterScanning());
		pCCPrintParam->SetPrintingNumber(m_CCProperty.GetPrintingNumber());
	}
}

// Check scanning process
int CApp::ScanCheck(void)
{
	int nResult = SUCCESS;

	m_byExecType = EXEC_SCAN_CHECK;

	// Set MF_BASE01 structure
	nResult = m_CCScan.SetBaseSetting();
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	// Set scanning parameters for check scanning
	nResult = m_CCScan.SetScanSetting();
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	// Set MICR parameters
	nResult = m_CCMicr.SetMicrSetting();
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	// Set MF_PROCESS01 structure
	nResult = m_CCOption.SetProcessSetting();
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	// Start check scanning
	nResult = m_CCScan.ScanFunction();
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	return nResult;
}

// Card scanning process
int CApp::ScanCard(void)
{
	int nResult = SUCCESS;

	m_byExecType = EXEC_SCAN_CARD;

	// Clear MF_MICR strucuture
	nResult = m_CCMicr.ClearMicrSetting();
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	// Set MF_BASE01 strucuture
	nResult = m_CCScan.SetBaseSetting();
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	// Set card scanning conditions
	nResult = m_CCScan.SetScanSetting();
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	// Start card scanning
	nResult = m_CCScan.ScanFunction();
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	return nResult;
}

// Get MICR data
int CApp::GetMicr(CMicrResult* pCCMicrResult)
{
	return m_CCMicr.GetMicr(pCCMicrResult);
}

// Get scanned image
int CApp::GetScanImage(CImageResult* pCCImageResult)
{
	return m_CCImage.GetScanImage(pCCImageResult);
}

// Get scanned image of printed cashiers check
int CApp::GetCashiersCheckImage(CImageResult* pCCImageResult)
{
	return m_CCImage.GetCashiersCheckImage(pCCImageResult);
}

// Error recovery
int CApp::CancelError(void)
{
	return BiCancelError(m_CCProperty.GetHandle());
}

// Get device status
int CApp::GetDeviceStatus(LPDWORD lpStatus)
{
	return BiGetStatus(m_CCProperty.GetHandle(), lpStatus);
}

// Cancel scanning
int CApp::CancelScan(void)
{
	return m_CCScan.CancelScan();
}

// MICR head cleaning
int CApp::CleaningMicr(void)
{
	return m_CCMicr.CleaningMicr();
}

// Get functions of the API
void CApp::GetAPIProcAddress(void)
{
	HINSTANCE hLibrary = m_CCProperty.GetLibrary();

	BiOpenMonPrinter				= (DLL_BiOpenMonPrinter)				GetProcAddress(hLibrary, "BiOpenMonPrinter");
	BiCloseMonPrinter				= (DLL_BiCloseMonPrinter)				GetProcAddress(hLibrary, "BiCloseMonPrinter");
	BiSCNMICRSetStatusBackFunction	= (DLL_BiSCNMICRSetStatusBackFunction)	GetProcAddress(hLibrary, "BiSCNMICRSetStatusBackFunction");
	BiSCNMICRCancelStatusBack		= (DLL_BiSCNMICRCancelStatusBack)		GetProcAddress(hLibrary, "BiSCNMICRCancelStatusBack");
	BiCancelError					= (DLL_BiCancelError)					GetProcAddress(hLibrary, "BiCancelError");
	BiGetStatus						= (DLL_BiGetStatus)						GetProcAddress(hLibrary, "BiGetStatus");
	// Step 5
	BiGetStatus						= (DLL_BiGetStatus)						GetProcAddress(hLibrary, "BiGetStatus");
	BiGetInkStatus					= (DLL_BiGetInkStatus)					GetProcAddress(hLibrary, "BiGetInkStatus");

	BiStartEndorsementSetStatusBackFunction = (DLL_BiStartEndorsementSetStatusBackFunction)	GetProcAddress(hLibrary, "BiStartEndorsementSetStatusBackFunction");
	BiStartEndorsementCancelStatusBack		= (DLL_BiStartEndorsementCancelStatusBack)		GetProcAddress(hLibrary, "BiStartEndorsementCancelStatusBack");
	BiEndEndorsementSetStatusBackFunction	= (DLL_BiEndEndorsementSetStatusBackFunction)	GetProcAddress(hLibrary, "BiEndEndorsementSetStatusBackFunction");
	BiEndEndorsementCancelStatusBack		= (DLL_BiEndEndorsementCancelStatusBack)		GetProcAddress(hLibrary, "BiEndEndorsementCancelStatusBack");
}

// Callback function for notifying scanning status
int CApp::cbScanStatus(DWORD dwTransactionNumber, WORD wMainStatus, WORD wSubStatus, LPSTR pPortName)
{
	return g_pCCApp->ScanStatus(dwTransactionNumber, wMainStatus, wSubStatus, pPortName);
}

int CApp::ScanStatus(DWORD dwTransactionNumber, WORD wMainStatus, WORD wSubStatus, LPSTR pPortName)
{
	if(wSubStatus != SUCCESS)
	{
		m_pFuncProcessErrorCB( static_cast<short>(wSubStatus) );
	}

	// Step 2
	if(wMainStatus == MF_DATARECEIVE_DONE)
	{
		m_CCProperty.SetTransactionNumber(dwTransactionNumber);

		// Step 3
		if( m_byExecType == EXEC_SCAN_CHECK )
		{
			// Notify callback for MICR data
			m_pFuncMicrCB();
		}
		// Notify callback for image data
		m_pFuncImageCB();
	}

	return 0;
}

////////////////
//// Step 5 ////
////////////////

// Register callback for cashiers check printing
void CApp::SetPrintCashiersCheckCallback(PRINT_CASHERSCHECK_BACK_FUNC pCashierscheckCB)
{
	m_pFuncCashierscheckCB = pCashierscheckCB;
}

// Register callback for receipt printing
void CApp::SetPrintReceiptCallback(PRINT_RECEIPT_BACK_FUNC pReceiptCB)
{
	m_pFuncReceiptCB = pReceiptCB;
}

// Get ink status
int CApp::GetInkStatus(LPWORD lpStatus)
{
	return BiGetInkStatus(m_CCProperty.GetHandle(), lpStatus);
}

// Callback function for notifying endorsement data acceptance
int CApp::cbStartEndorsement(unsigned long ulTransactionNumber, LPSTR lpcPortName)
{
	return g_pCCApp->StartEndorsement(ulTransactionNumber, lpcPortName);
}

int CApp::StartEndorsement(unsigned long ulTransactionNumber, LPSTR lpcPortName)
{
	int nResult = SUCCESS;

	if(m_byExecType == EXEC_PRINT_CAHSHERSCHECK)
	{
		m_pFuncCashierscheckCB();
	}
	else if(m_byExecType == EXEC_PRINT_RECEIPT)
	{
		m_pFuncReceiptCB(ulTransactionNumber);
	}

	return 0;
}

// Callback function for notifying endorsement data close
int CApp::cbEndEndorsement(unsigned long ulTransactionNumber, LPSTR lpcPortName)
{
	return g_pCCApp->EndEndorsement(ulTransactionNumber, lpcPortName);
}

int CApp::EndEndorsement(unsigned long ulTransactionNumber, LPSTR lpcPortName)
{
	/* After this callback is executed, all endorsement data is discarded */
	return 0;
}

// Printing on roll paper
int CApp::PrintRollpaper(void)
{
	// S9000 only
	int nResult = SUCCESS;

	nResult = m_CCPrint.PrintRollpaper();

	return nResult;
}

// Printing on cut sheet paper
int CApp::PrintReceipt(void)
{
	int nResult = SUCCESS;

	m_byExecType = EXEC_PRINT_RECEIPT;

	nResult = m_CCPrint.PrintReceipt();

	return nResult;
}

// Set printing data on cut sheet paper
int CApp::PrintReceiptFunction(unsigned long ulTransactionNumber)
{
	int nResult = SUCCESS;

	nResult = m_CCPrint.PrintReceiptFunction(ulTransactionNumber);

	return nResult;
}

// Printing cashiers check
int CApp::PrintCashierscheck(void)
{
	int		nResult = SUCCESS;
	bool	bScanning = true;

	m_byExecType = EXEC_PRINT_CAHSHERSCHECK;

	nResult = m_CCPrint.SetCashierscheckPrintSetting();
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	nResult = m_CCPrint.SetCashierscheckBaseSetting();
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	bScanning = m_CCProperty.GetAfterScanning();
	if(bScanning == true)
	{
		nResult = m_CCPrint.SetCashierscheckScanSetting();
		if(nResult != SUCCESS)
		{
			return nResult;
		}
	}

	nResult = m_CCPrint.PrintCashierscheck();
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	return nResult;
}

// Set print data for cashiers check
int CApp::PrintCacherscheckFunction(void)
{
	int nResult = SUCCESS;

	nResult = m_CCPrint.PrintCashierscheckFunction();

	return nResult;
}

// Check if the roll paper is supported as print station.
bool CApp::IsRollPaperSupported(void)
{
	return m_CCPrint.IsRollPaperSupported();
}
