#include "stdafx.h"
#include "App.h"

CApp*	g_pCCApp;

CApp::CApp(void)
{
	m_pFuncMicrCB				= NULL;
	m_pFuncImageCB				= NULL;
	m_pFuncProcessErrorCB		= NULL;
	g_pCCApp					= this;
	m_byExecType				= EXEC_SCAN_CHECK;
	// Step 7
	m_pFuncIqaCB				= NULL;
	m_pFuncConfirmationCB		= NULL;
}

CApp::~CApp(void)
{
	ExitDevice();
	g_pCCApp = NULL;
}

// Initialize
int CApp::InitDevice(void)
{
	int nResult = SUCCESS;

	if(m_CCProperty.GetLibrary() == NULL)
	{
		// Load the TM-S9000 driver
		HINSTANCE hLibrary = ::LoadLibrary(_T("EpsStmApiWrapper.dll"));
		if(hLibrary == NULL)
		{
			return ERR_LOAD_MODULE;
		}
		m_CCProperty.SetLibrary(hLibrary);
	}

	GetAPIProcAddress();

	if(m_CCProperty.GetHandle() <= 0)
	{
		// Connect to the device
		int nHandle = BiOpenMonPrinter(TYPE_PRINTER, "TM-S9000U,TM-S2000U");
		if(nHandle <= 0)
		{
			return nHandle;
		}

		m_CCProperty.SetHandle(nHandle);
	}

	m_CCScan.Init(&m_CCProperty);
	m_CCMicr.Init(&m_CCProperty);
	m_CCOption.Init(&m_CCProperty);
	// Step 2
	m_CCImage.Init(&m_CCProperty);
	// Step 7
	m_CCIqa.Init(&m_CCProperty);

	// Register callback function for notification of the reading status
	nResult = BiSCNMICRSetStatusBackFunction(m_CCProperty.GetHandle(), cbScanStatus);
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	return nResult;
}

// Post process
int CApp::ExitDevice(void)
{
	int nResult = SUCCESS;

	if(m_CCProperty.GetHandle() > 0)
	{
		// Cancel the reading status callback
		nResult = BiSCNMICRCancelStatusBack(m_CCProperty.GetHandle());
		if(nResult != SUCCESS)
		{
			return nResult;
		}

		// Disconnect the device
		nResult = BiCloseMonPrinter(m_CCProperty.GetHandle());
		if(nResult != SUCCESS)
		{
			return nResult;
		}
		m_CCProperty.SetHandle(0);
	}

	if(m_CCProperty.GetLibrary() != NULL)
	{
		// Unload the driver
		::FreeLibrary(m_CCProperty.GetLibrary());
		m_CCProperty.SetLibrary(NULL);
	}

	return nResult;
}

// Register callback function for notification of the MICR data
void CApp::SetMicrCallback(MICR_IMAGE_BACK_FUNC pMicrCB)
{
	m_pFuncMicrCB = pMicrCB;
}

// Register callback function for notification of the image data
void CApp::SetImageCallback(MICR_IMAGE_BACK_FUNC pImageCB)
{
	m_pFuncImageCB = pImageCB;
}

// Register callback function for notification of error during image acquisition
void CApp::SetProcessErrorCallback(PROCESS_ERROR_BACK_FUNC pProcessErrorCB)
{
	m_pFuncProcessErrorCB = pProcessErrorCB;
}

// Set MICR settings
void CApp::SetMicrParam(CMicrParam* pCCMicrParam)
{
	if(pCCMicrParam != NULL)
	{
		m_CCProperty.SetFont(pCCMicrParam->GetFont());
		m_CCProperty.SetParsing(pCCMicrParam->GetParsing());
		m_CCProperty.SetClearSpace(pCCMicrParam->GetClearSpace());
	}
}

// Get MICR settings
void CApp::GetMicrParam(CMicrParam* pCCMicrParam)
{
	if(pCCMicrParam != NULL)
	{
		pCCMicrParam->SetFont(m_CCProperty.GetFont());
		pCCMicrParam->SetParsing(m_CCProperty.GetParsing());
		pCCMicrParam->SetClearSpace(m_CCProperty.GetClearSpace());
	}
}

// Set scanning parameters
void CApp::SetScanParam(CScanParam* pCCScanParam)
{
	if(pCCScanParam != NULL)
	{
		m_CCProperty.SetScanMedia(pCCScanParam->GetScanMedia());
		m_CCProperty.SetImageChannel(pCCScanParam->GetImageChannel());
		m_CCProperty.SetRGBColorDepth(pCCScanParam->GetRGBColorDepth());
		m_CCProperty.SetIRColorDepth(pCCScanParam->GetIRColorDepth());
		m_CCProperty.SetResolution(pCCScanParam->GetResolution());
	}
}

// Get scanning parameters
void CApp::GetScanParam(CScanParam* pCCScanParam)
{
	if(pCCScanParam != NULL)
	{
		pCCScanParam->SetScanMedia(m_CCProperty.GetScanMedia());
		pCCScanParam->SetImageChannel(m_CCProperty.GetImageChannel());
		pCCScanParam->SetRGBColorDepth(m_CCProperty.GetRGBColorDepth());
		pCCScanParam->SetIRColorDepth(m_CCProperty.GetIRColorDepth());
		pCCScanParam->SetResolution(m_CCProperty.GetResolution());
	}
}

// Check scanning process
int CApp::ScanCheck(void)
{
	int nResult = SUCCESS;

	m_byExecType = EXEC_SCAN_CHECK;

	// Set MF_BASE01 structure
	nResult = m_CCScan.SetBaseSetting();
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	// Set check scanning conditions
	nResult = m_CCScan.SetScanSetting();
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	// Set MICR reading conditions
	nResult = m_CCMicr.SetMicrSetting();
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	// Step 7
	// Set IQA test conditions
	nResult = m_CCIqa.SetIqaSetting();
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	// Set MF_PROCESS01 structure
	nResult = m_CCOption.SetProcessSetting();
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	// Set water fall mode
	nResult = m_CCOption.SetWaterfallMode();
	if(nResult != SUCCESS)
	{
		return nResult;
	}
	
	// Start check scanning
	nResult = m_CCScan.ScanFunction();
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	return nResult;
}

// Card scanning process
int CApp::ScanCard(void)
{
	int nResult = SUCCESS;

	m_byExecType = EXEC_SCAN_CARD;

	// Clear MF_MICR strucuture
	nResult = m_CCMicr.ClearMicrSetting();
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	// Set MF_BASE01 strucuture
	nResult = m_CCScan.SetBaseSetting();
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	// Set card scanning conditions
	nResult = m_CCScan.SetScanSetting();
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	// Start card scanning
	nResult = m_CCScan.ScanFunction();
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	return nResult;
}

// Get MICR data
int CApp::GetMicr(CMicrResult* pCCMicrResult)
{
	return m_CCMicr.GetMicr(pCCMicrResult);
}

// Get scanned image
int CApp::GetScanImage(CImageResult* pCCImageResult)
{
	return m_CCImage.GetScanImage(pCCImageResult);
}

// Error recovery
int CApp::CancelError(void)
{
	return BiCancelError(m_CCProperty.GetHandle());
}

// Get device status
int CApp::GetDeviceStatus(LPDWORD lpStatus)
{
	return BiGetStatus(m_CCProperty.GetHandle(), lpStatus);
}

// Cancel scanning
int CApp::CancelScan(void)
{
	return m_CCScan.CancelScan();
}

// MICR head cleaning
int CApp::CleaningMicr(void)
{
	return m_CCMicr.CleaningMicr();
}

// Get functions of the API
void CApp::GetAPIProcAddress(void)
{
	HINSTANCE hLibrary = m_CCProperty.GetLibrary();

	BiOpenMonPrinter				= (DLL_BiOpenMonPrinter)				GetProcAddress(hLibrary, "BiOpenMonPrinter");
	BiCloseMonPrinter				= (DLL_BiCloseMonPrinter)				GetProcAddress(hLibrary, "BiCloseMonPrinter");
	BiSCNMICRSetStatusBackFunction	= (DLL_BiSCNMICRSetStatusBackFunction)	GetProcAddress(hLibrary, "BiSCNMICRSetStatusBackFunction");
	BiSCNMICRCancelStatusBack		= (DLL_BiSCNMICRCancelStatusBack)		GetProcAddress(hLibrary, "BiSCNMICRCancelStatusBack");
	BiCancelError					= (DLL_BiCancelError)					GetProcAddress(hLibrary, "BiCancelError");
	BiGetStatus						= (DLL_BiGetStatus)						GetProcAddress(hLibrary, "BiGetStatus");
}

// Callback function for notifying scanning status
int CApp::cbScanStatus(DWORD dwTransactionNumber, WORD wMainStatus, WORD wSubStatus, LPSTR pPortName)
{
	return g_pCCApp->ScanStatus(dwTransactionNumber, wMainStatus, wSubStatus, pPortName);
}

int CApp::ScanStatus(DWORD dwTransactionNumber, WORD wMainStatus, WORD wSubStatus, LPSTR pPortName)
{
	if(wSubStatus != SUCCESS)
	{
		m_pFuncProcessErrorCB( static_cast<short>(wSubStatus) );
	}

	// Step 2
	if(wMainStatus == MF_DATARECEIVE_DONE)
	{
		m_CCProperty.SetTransactionNumber(dwTransactionNumber);

		// Step 3
		if( m_byExecType == EXEC_SCAN_CHECK )
		{
			// Notify callback for MICR data
			m_pFuncMicrCB();

			// Step 7
			// Notify callback for IQA test
			m_pFuncIqaCB();

			if( m_CCProperty.GetActivation() == MF_ACTIVATE_MODE_CONFIRMATION )
			{
				// notify callback for scanning in confirmatio mode
				m_pFuncConfirmationCB();
			}
		}
		// Notify callback for image data
		m_pFuncImageCB();
	}

	return 0;
}

////////////////
//// Step 7 ////
////////////////

// Register callback for obtaining IQA result
void CApp::SetIqaCallback(IQA_RESULT_BACK_FUNC pIqaCB)
{
	m_pFuncIqaCB = pIqaCB;
}

// Register callback for scanning in confirmation mode
void CApp::SetConfirmationCallback(CONFIRMATION_BACK_FUNC pConfirmationCB)
{
	m_pFuncConfirmationCB = pConfirmationCB;
}

// Set IQA test conditions
void CApp::SetIqaParam(CIqaParam* pCCIqaParam)
{
	if( pCCIqaParam != NULL)
	{
		m_CCProperty.SetIqaDetect(pCCIqaParam->GetIqaDetect());
		m_CCProperty.SetIqaCancel(pCCIqaParam->GetIqaCancel());
		m_CCProperty.SetIqaEjectPocket(pCCIqaParam->GetIqaEjectPocket());
		m_CCProperty.SetIqaColorDepth(pCCIqaParam->GetIqaColorDepth());
		m_CCProperty.SetUnderSize(pCCIqaParam->GetUnderSize());
		m_CCProperty.SetOverSize(pCCIqaParam->GetOverSize());
		m_CCProperty.SetMinCompressed(pCCIqaParam->GetMinCompressed());
		m_CCProperty.SetMaxCompressed(pCCIqaParam->GetMaxCompressed());
		m_CCProperty.SetFrontRearImageMismatch(pCCIqaParam->GetFrontRearImageMismatch());
		m_CCProperty.SetImageTooLight(pCCIqaParam->GetImageTooLight());
		m_CCProperty.SetImageTooDark(pCCIqaParam->GetImageTooDark());
		m_CCProperty.SetHorizontalStreaksPresent(pCCIqaParam->GetHorizontalStreaksPresent());
		m_CCProperty.SetSpotNoise(pCCIqaParam->GetSpotNoise());
		m_CCProperty.SetOutOfFocus(pCCIqaParam->GetOutOfFocus());
		m_CCProperty.SetFoldedTornCorner(pCCIqaParam->GetFoldedTornCorner());
		m_CCProperty.SetFoldedTornEdge(pCCIqaParam->GetFoldedTornEdge());
		m_CCProperty.SetFramingError(pCCIqaParam->GetFramingError());
		m_CCProperty.SetExcessiveSkew(pCCIqaParam->GetExcessiveSkew());
		m_CCProperty.SetCarbonStripDetection(pCCIqaParam->GetCarbonStripDetection());
		m_CCProperty.SetPiggyback(pCCIqaParam->GetPiggyback());
	}
}

// Set scanning option settings
void CApp::SetOptionParam(COptionParam* pCCOptionParam)
{
	if(pCCOptionParam != NULL)
	{
		m_CCProperty.SetActivation(pCCOptionParam->GetActivation());
		m_CCProperty.SetWaterfall(pCCOptionParam->GetWaterfall());
	}
}

// Set settings in confirmation mode
void CApp::SetConfirmationParam(COptionParam *pCCOptionParam)
{
	if(pCCOptionParam != NULL)
	{
		m_CCProperty.SetConfirmationEject(pCCOptionParam->GetConfirmationEject());
		m_CCProperty.SetConfirmationCancel(pCCOptionParam->GetConfirmationCancel());
	}
}

// Obtain IQA result
void CApp::GetIqaParam(CIqaParam* pCCIqaParam)
{
	if(pCCIqaParam != NULL)
	{
		pCCIqaParam->SetIqaDetect(m_CCProperty.GetIqaDetect());
		pCCIqaParam->SetIqaCancel(m_CCProperty.GetIqaCancel());
		pCCIqaParam->SetIqaEjectPocket(m_CCProperty.GetIqaEjectPocket());
		pCCIqaParam->SetIqaColorDepth(m_CCProperty.GetIqaColorDepth());
		pCCIqaParam->SetUnderSize(m_CCProperty.GetUnderSize());
		pCCIqaParam->SetOverSize(m_CCProperty.GetOverSize());
		pCCIqaParam->SetMinCompressed(m_CCProperty.GetMinCompressed());
		pCCIqaParam->SetMaxCompressed(m_CCProperty.GetMaxCompressed());
		pCCIqaParam->SetFrontRearImageMismatch(m_CCProperty.GetFrontRearImageMismatch());
		pCCIqaParam->SetImageTooLight(m_CCProperty.GetImageTooLight());
		pCCIqaParam->SetImageTooDark(m_CCProperty.GetImageTooDark());
		pCCIqaParam->SetHorizontalStreaksPresent(m_CCProperty.GetHorizontalStreaksPresent());
		pCCIqaParam->SetSpotNoise(m_CCProperty.GetSpotNoise());
		pCCIqaParam->SetOutOfFocus(m_CCProperty.GetOutOfFocus());
		pCCIqaParam->SetFoldedTornCorner(m_CCProperty.GetFoldedTornCorner());
		pCCIqaParam->SetFoldedTornEdge(m_CCProperty.GetFoldedTornEdge());
		pCCIqaParam->SetFramingError(m_CCProperty.GetFramingError());
		pCCIqaParam->SetExcessiveSkew(m_CCProperty.GetExcessiveSkew());
		pCCIqaParam->SetCarbonStripDetection(m_CCProperty.GetCarbonStripDetection());
		pCCIqaParam->SetPiggyback(m_CCProperty.GetPiggyback());
	}
}

// Get scanning option settings
void CApp::GetOptionParam(COptionParam* pCCOptionParam)
{
	if(pCCOptionParam != NULL)
	{
		pCCOptionParam->SetActivation(m_CCProperty.GetActivation());
		pCCOptionParam->SetWaterfall(m_CCProperty.GetWaterfall());
	}
}

// Get IQA test result
int CApp::GetIqaResult(CIqaResult *pCCIqaResult)
{
	return m_CCIqa.GetIqa(pCCIqaResult);
}

// Perform confirmation mode processing
int CApp::ConfirmationFunction(void)
{
	return m_CCOption.ConfirmationFunction();
}

