#include "stdafx.h"
#include "Iqa.h"

CIqa::CIqa(void)
{
	m_pCProperty = NULL;
}

CIqa::~CIqa(void)
{
}

int CIqa::Init(CProperty_1* pCCProperty)
{
	int nResult = SUCCESS;

	m_pCProperty = pCCProperty;

	GetAPIProcAddress();

	return nResult;
}

// Set IQA test conditions
int CIqa::SetIqaSetting(void)
{
	int nResult = SUCCESS;

	nResult = BiSCNMICRFunctionContinuously(m_pCProperty->GetHandle(), m_pCProperty->GetMfIqa(), MF_GET_IQA_DEFAULT);
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	m_pCProperty->GetMfIqa()->iSize				= sizeof(MF_IQA);
	m_pCProperty->GetMfIqa()->iVersion			= MF_IQA_VERSION;
	m_pCProperty->GetMfIqa()->bErrorSelect		= m_pCProperty->GetIqaDetect();
	m_pCProperty->GetMfIqa()->bErrorEject		= m_pCProperty->GetIqaEjectPocket();
	m_pCProperty->GetMfIqa()->sResolution		= m_pCProperty->GetResolution();

	if( m_pCProperty->GetIqaDetect() == MF_ERROR_SELECT_DETECT )
	{
		m_pCProperty->GetMfIqa()->bCancel			= m_pCProperty->GetIqaCancel();
		m_pCProperty->GetMfIqa()->bColorDepth		= m_pCProperty->GetIqaColorDepth();
		m_pCProperty->GetMfIqa()->bUndersize		= m_pCProperty->GetUnderSize();
		m_pCProperty->GetMfIqa()->bOversize			= m_pCProperty->GetOverSize();
		m_pCProperty->GetMfIqa()->bMincompressed	= m_pCProperty->GetMinCompressed();
		m_pCProperty->GetMfIqa()->bMaxcompressed	= m_pCProperty->GetMaxCompressed();
		m_pCProperty->GetMfIqa()->bFront_rear		= m_pCProperty->GetFrontRearImageMismatch();
		m_pCProperty->GetMfIqa()->bToolight			= m_pCProperty->GetImageTooLight();
		m_pCProperty->GetMfIqa()->bToodark			= m_pCProperty->GetImageTooDark();
		m_pCProperty->GetMfIqa()->bStreaks			= m_pCProperty->GetHorizontalStreaksPresent();
		m_pCProperty->GetMfIqa()->bNoise			= m_pCProperty->GetSpotNoise();
		m_pCProperty->GetMfIqa()->bFocus			= m_pCProperty->GetOutOfFocus();
		m_pCProperty->GetMfIqa()->bCorners			= m_pCProperty->GetFoldedTornCorner();
		m_pCProperty->GetMfIqa()->bEdges			= m_pCProperty->GetFoldedTornEdge();
		m_pCProperty->GetMfIqa()->bFraming			= m_pCProperty->GetFramingError();
		m_pCProperty->GetMfIqa()->bSkew				= m_pCProperty->GetExcessiveSkew();
		m_pCProperty->GetMfIqa()->bCarbon			= m_pCProperty->GetCarbonStripDetection();
		m_pCProperty->GetMfIqa()->bPiggyback		= m_pCProperty->GetPiggyback();

		if(m_pCProperty->GetIqaColorDepth() == EPS_BI_SCN_1BIT)
		{
			m_pCProperty->GetMfIqa()->bImageFormat = EPS_BI_SCN_TIFF;
		}
		else
		{
			m_pCProperty->GetMfIqa()->bImageFormat = EPS_BI_SCN_JPEGNORMAL;
		}
	}

	nResult = BiSCNMICRFunctionContinuously(m_pCProperty->GetHandle(), m_pCProperty->GetMfIqa(), MF_SET_IQA_PARAM);
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	return nResult;
}

// Obtain IQA test result
int CIqa::GetIqa(CIqaResult* pCCIqaResult)
{
	int nResult = SUCCESS;

	if(pCCIqaResult == NULL)
	{
		return -1;
	}

	nResult = BiGetIqaResult(m_pCProperty->GetHandle(), m_pCProperty->GetTransactionNumber(), m_pCProperty->GetMfIqaResult());
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	pCCIqaResult->SetUnderSize(m_pCProperty->GetMfIqaResult()->stUnderSize.bResult);
	pCCIqaResult->SetOverSize(m_pCProperty->GetMfIqaResult()->stOverSize.bResult);
	pCCIqaResult->SetMinCompressed(m_pCProperty->GetMfIqaResult()->stMinCompressedImageSize.bResult);
	pCCIqaResult->SetMaxCompressed(m_pCProperty->GetMfIqaResult()->stMaxCompressedImageSize.bResult);
	pCCIqaResult->SetFrontRearImageMismatch(m_pCProperty->GetMfIqaResult()->stFrontRearImageMismatch.bResult);
	pCCIqaResult->SetImageTooLight(m_pCProperty->GetMfIqaResult()->stImageTooLight.bResult);
	pCCIqaResult->SetImageTooDark(m_pCProperty->GetMfIqaResult()->stImageTooDark.bResult);
	pCCIqaResult->SetHorizontalStreaksPresent(m_pCProperty->GetMfIqaResult()->stHorizontalStreaksPresent.bResult);
	pCCIqaResult->SetSpotNosise(m_pCProperty->GetMfIqaResult()->stExcessiveSpotNoise.bResult);
	pCCIqaResult->SetOutOfFocus(m_pCProperty->GetMfIqaResult()->stImageOutOfFocus.bResult);
	pCCIqaResult->SetFoldedTornCorner(m_pCProperty->GetMfIqaResult()->stFoldedTornDocCorners.bResult);
	pCCIqaResult->SetFoldedTornEdge(m_pCProperty->GetMfIqaResult()->stFoldedTornDocEdges.bResult);
	pCCIqaResult->SetFramingError(m_pCProperty->GetMfIqaResult()->stDocFramingError.bResult);
	pCCIqaResult->SetExcessiveSkew(m_pCProperty->GetMfIqaResult()->stExcessiveDocSkew.bResult);
	pCCIqaResult->SetCarbonStripDetection(m_pCProperty->GetMfIqaResult()->stCarbonStripDetection.bResult);
	pCCIqaResult->SetPiggyback(m_pCProperty->GetMfIqaResult()->stPiggyBack.bResult);

	return nResult;
}

// Get functions of the API
void CIqa::GetAPIProcAddress(void)
{
	HINSTANCE hLibrary = m_pCProperty->GetLibrary();

	BiSCNMICRFunctionContinuously	= (DLL_BiSCNMICRFunctionContinuously)	GetProcAddress(hLibrary, "BiSCNMICRFunctionContinuously");
	BiGetIqaResult					= (DLL_BiGetIqaResult)					GetProcAddress(hLibrary, "BiGetIqaResult");
}
