#include "stdafx.h"
#include "App.h"

CApp*	g_pCCApp;

CApp::CApp(void)
{
	m_pFuncMicrCB				= NULL;
	m_pFuncImageCB				= NULL;
	m_pFuncProcessErrorCB		= NULL;
	m_pFuncDeviceStatusCB		= NULL;
	m_pFuncInkStatusCB			= NULL;
	g_pCCApp					= this;
	m_byExecType				= EXEC_SCAN_CHECK;
}

CApp::~CApp(void)
{
	ExitDevice();
	g_pCCApp = NULL;
}

// Initialize
int CApp::InitDevice(void)
{
	int nResult = SUCCESS;

	if(m_CCProperty.GetLibrary() == NULL)
	{
		// Load the TM-S9000 driver
		HINSTANCE hLibrary = ::LoadLibrary(_T("EpsStmApiWrapper.dll"));
		if(hLibrary == NULL)
		{
			return ERR_LOAD_MODULE;
		}
		m_CCProperty.SetLibrary(hLibrary);
	}

	GetAPIProcAddress();

	if(m_CCProperty.GetHandle() <= 0)
	{
		// Connect to the device
		int nHandle = BiOpenMonPrinter(TYPE_PRINTER, "TM-S9000U,TM-S2000U");
		if(nHandle <= 0)
		{
			return nHandle;
		}

		m_CCProperty.SetHandle(nHandle);
	}

	m_CCScan.Init(&m_CCProperty);
	m_CCMicr.Init(&m_CCProperty);
	m_CCOption.Init(&m_CCProperty);
	// Step 2
	m_CCImage.Init(&m_CCProperty);
	// Step 8
	m_CCBarcode.Init(&m_CCProperty);
	m_CCInfo.Init(&m_CCProperty);

	// Register callback function for notification of the reading status
	nResult = BiSCNMICRSetStatusBackFunction(m_CCProperty.GetHandle(), cbScanStatus);
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	// Step 8
	// Register callback function for notification of the device status
	nResult = BiSetStatusBackFunction(m_CCProperty.GetHandle(), cbStatus);
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	// Register callback function for notification of ink status
	nResult = BiSetInkStatusBackFunction(m_CCProperty.GetHandle(), cbInkStatus);
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	return nResult;
}

// Post process
int CApp::ExitDevice(void)
{
	int nResult = SUCCESS;

	if(m_CCProperty.GetHandle() > 0)
	{
		// Cancel the reading status callback
		nResult = BiSCNMICRCancelStatusBack(m_CCProperty.GetHandle());
		if(nResult != SUCCESS)
		{
			return nResult;
		}

		// Step 8
		// Cancel the device status callback
		nResult = BiCancelStatusBack(m_CCProperty.GetHandle());
		if(nResult != SUCCESS)
		{
			return nResult;
		}

		// Cancel the ink status callback
		nResult = BiCancelInkStatusBack(m_CCProperty.GetHandle());
		if(nResult != SUCCESS)
		{
			return nResult;
		}

		// Disconnect the device
		nResult = BiCloseMonPrinter(m_CCProperty.GetHandle());
		if(nResult != SUCCESS)
		{
			return nResult;
		}
		m_CCProperty.SetHandle(0);
	}

	if(m_CCProperty.GetLibrary() != NULL)
	{
		// Unload the driver
		::FreeLibrary(m_CCProperty.GetLibrary());
		m_CCProperty.SetLibrary(NULL);
	}

	return nResult;
}

// Register callback function for notification of the MICR data
void CApp::SetMicrCallback(MICR_IMAGE_BACK_FUNC pMicrCB)
{
	m_pFuncMicrCB = pMicrCB;
}

// Register callback function for notification of the image data
void CApp::SetImageCallback(MICR_IMAGE_BACK_FUNC pImageCB)
{
	m_pFuncImageCB = pImageCB;
}

// Register callback function for notification of error during image acquisition
void CApp::SetProcessErrorCallback(PROCESS_ERROR_BACK_FUNC pProcessErrorCB)
{
	m_pFuncProcessErrorCB = pProcessErrorCB;
}

// Set MICR settings
void CApp::SetMicrParam(CMicrParam* pCCMicrParam)
{
	if(pCCMicrParam != NULL)
	{
		m_CCProperty.SetFont(pCCMicrParam->GetFont());
		m_CCProperty.SetParsing(pCCMicrParam->GetParsing());
		m_CCProperty.SetClearSpace(pCCMicrParam->GetClearSpace());
	}
}

// Get MICR settings
void CApp::GetMicrParam(CMicrParam* pCCMicrParam)
{
	if(pCCMicrParam != NULL)
	{
		pCCMicrParam->SetFont(m_CCProperty.GetFont());
		pCCMicrParam->SetParsing(m_CCProperty.GetParsing());
		pCCMicrParam->SetClearSpace(m_CCProperty.GetClearSpace());
	}
}

// Set scanning parameters
void CApp::SetScanParam(CScanParam* pCCScanParam)
{
	if(pCCScanParam != NULL)
	{
		m_CCProperty.SetScanMedia(pCCScanParam->GetScanMedia());
		m_CCProperty.SetImageChannel(pCCScanParam->GetImageChannel());
		m_CCProperty.SetRGBColorDepth(pCCScanParam->GetRGBColorDepth());
		m_CCProperty.SetIRColorDepth(pCCScanParam->GetIRColorDepth());
		m_CCProperty.SetResolution(pCCScanParam->GetResolution());
	}
}

// Get scanning parameters
void CApp::GetScanParam(CScanParam* pCCScanParam)
{
	if(pCCScanParam != NULL)
	{
		pCCScanParam->SetScanMedia(m_CCProperty.GetScanMedia());
		pCCScanParam->SetImageChannel(m_CCProperty.GetImageChannel());
		pCCScanParam->SetRGBColorDepth(m_CCProperty.GetRGBColorDepth());
		pCCScanParam->SetIRColorDepth(m_CCProperty.GetIRColorDepth());
		pCCScanParam->SetResolution(m_CCProperty.GetResolution());
	}
}

// Check scanning process
int CApp::ScanCheck(void)
{
	int nResult = SUCCESS;

	m_byExecType = EXEC_SCAN_CHECK;

	// Set MF_BASE01 structure
	nResult = m_CCScan.SetBaseSetting();
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	// Set check scanning conditions
	nResult = m_CCScan.SetScanSetting();
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	// Set MICR reading conditions
	nResult = m_CCMicr.SetMicrSetting();
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	// Step 8
	// Set barcode recoginition conditions
	nResult = m_CCBarcode.SetBarcodeSetting();
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	// Set MF_PROCESS01 structure
	nResult = m_CCOption.SetProcessSetting();
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	// Start check scanning
	nResult = m_CCScan.ScanFunction();
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	return nResult;
}

// Card scanning process
int CApp::ScanCard(void)
{
	int nResult = SUCCESS;

	m_byExecType = EXEC_SCAN_CARD;

	// Clear MF_MICR strucuture
	nResult = m_CCMicr.ClearMicrSetting();
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	// Set MF_BASE01 strucuture
	nResult = m_CCScan.SetBaseSetting();
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	// Set card scanning conditions
	nResult = m_CCScan.SetScanSetting();
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	// Start card scanning
	nResult = m_CCScan.ScanFunction();
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	return nResult;
}

// Get MICR data
int CApp::GetMicr(CMicrResult* pCCMicrResult)
{
	return m_CCMicr.GetMicr(pCCMicrResult);
}

// Get scanned image
int CApp::GetScanImage(CImageResult* pCCImageResult)
{
	return m_CCImage.GetScanImage(pCCImageResult);
}

// Error recovery
int CApp::CancelError(void)
{
	return BiCancelError(m_CCProperty.GetHandle());
}

// Get device status
int CApp::GetDeviceStatus(LPDWORD lpStatus)
{
	return BiGetStatus(m_CCProperty.GetHandle(), lpStatus);
}

// Cancel scanning
int CApp::CancelScan(void)
{
	return m_CCScan.CancelScan();
}

// MICR head cleaning
int CApp::CleaningMicr(void)
{
	return m_CCMicr.CleaningMicr();
}

// Get functions of the API
void CApp::GetAPIProcAddress(void)
{
	HINSTANCE hLibrary = m_CCProperty.GetLibrary();

	BiOpenMonPrinter				= (DLL_BiOpenMonPrinter)				GetProcAddress(hLibrary, "BiOpenMonPrinter");
	BiCloseMonPrinter				= (DLL_BiCloseMonPrinter)				GetProcAddress(hLibrary, "BiCloseMonPrinter");
	BiSCNMICRSetStatusBackFunction	= (DLL_BiSCNMICRSetStatusBackFunction)	GetProcAddress(hLibrary, "BiSCNMICRSetStatusBackFunction");
	BiSCNMICRCancelStatusBack		= (DLL_BiSCNMICRCancelStatusBack)		GetProcAddress(hLibrary, "BiSCNMICRCancelStatusBack");
	BiCancelError					= (DLL_BiCancelError)					GetProcAddress(hLibrary, "BiCancelError");
	BiGetStatus						= (DLL_BiGetStatus)						GetProcAddress(hLibrary, "BiGetStatus");
	// Step 8
	BiSetStatusBackFunction			= (DLL_BiSetStatusBackFunction)			GetProcAddress(hLibrary, "BiSetStatusBackFunction");
	BiSetInkStatusBackFunction		= (DLL_BiSetInkStatusBackFunction)		GetProcAddress(hLibrary, "BiSetInkStatusBackFunction");
	BiCancelStatusBack				= (DLL_BiCancelStatusBack)				GetProcAddress(hLibrary, "BiCancelStatusBack");
	BiCancelInkStatusBack			= (DLL_BiCancelInkStatusBack)			GetProcAddress(hLibrary, "BiCancelInkStatusBack");
}

// Callback function for notifying scanning status
int CApp::cbScanStatus(DWORD dwTransactionNumber, WORD wMainStatus, WORD wSubStatus, LPSTR pPortName)
{
	return g_pCCApp->ScanStatus(dwTransactionNumber, wMainStatus, wSubStatus, pPortName);
}

int CApp::ScanStatus(DWORD dwTransactionNumber, WORD wMainStatus, WORD wSubStatus, LPSTR pPortName)
{
	if(wSubStatus != SUCCESS)
	{
		m_pFuncProcessErrorCB( static_cast<short>(wSubStatus) );
	}

	// Step 2
	if(wMainStatus == MF_DATARECEIVE_DONE)
	{
		m_CCProperty.SetTransactionNumber(dwTransactionNumber);

		// Step 3
		if( m_byExecType == EXEC_SCAN_CHECK )
		{
			// Notify callback for MICR data
			m_pFuncMicrCB();

			// Step 8
			// Notify callback for barcode data
			m_pFuncBarcodeCB();
		}
		// Notify callback for image data
		m_pFuncImageCB();
	}

	return 0;
}

////////////////
//// Step 8 ////
////////////////

// Set barcode recognition settings
void CApp::SetBarcodeParam(CBarcodeParam* pCCBarcodeParam)
{
	if(pCCBarcodeParam != NULL)
	{
		m_CCProperty.SetBarcodeDetect(pCCBarcodeParam->GetBarcodeDetect());
		m_CCProperty.SetBarcodeCancel(pCCBarcodeParam->GetBarcodeCancel());
		m_CCProperty.SetBarcodeEject(pCCBarcodeParam->GetBarcodeEject());
	}
}

// Get barcode recognition settings
void CApp::GetBarcodeParam(CBarcodeParam* pCCBarcodeParam)
{
	if(pCCBarcodeParam != NULL)
	{
		pCCBarcodeParam->SetBarcodeDetect(m_CCProperty.GetBarcodeDetect());
		pCCBarcodeParam->SetBarcodeCancel(m_CCProperty.GetBarcodeCancel());
		pCCBarcodeParam->SetBarcodeEject(m_CCProperty.GetBarcodeEject());
	}
}

// Set scanning option settings
void CApp::SetOptionParam(COptionParam* pCCOptionParam)
{
	if( pCCOptionParam != NULL)
	{
		m_CCProperty.SetPaperMisInsertionDetect(pCCOptionParam->GetPaperMisInsertionDetect());
		m_CCProperty.SetPaperMisInsertionCancel(pCCOptionParam->GetPaperMisInsertionCancel());
		m_CCProperty.SetPaperMisInsertionEject(pCCOptionParam->GetPaperMisInsertionEject());
		m_CCProperty.SetNoiseDetect(pCCOptionParam->GetNoiseDetect());
		m_CCProperty.SetNoiseCancel(pCCOptionParam->GetNoiseCancel());
		m_CCProperty.SetNoiseEject(pCCOptionParam->GetNoiseEject());
		m_CCProperty.SetDoubleFeedDetect(pCCOptionParam->GetDoubleFeedDetect());
		m_CCProperty.SetDoubleFeedCancel(pCCOptionParam->GetDoubleFeedCancel());
		m_CCProperty.SetDoubleFeedEject(pCCOptionParam->GetDoubleFeedEject());
		m_CCProperty.SetBaddataDetect(pCCOptionParam->GetBaddataDetect());
		m_CCProperty.SetBaddataCancel(pCCOptionParam->GetBaddataCancel());
		m_CCProperty.SetBaddataEject(pCCOptionParam->GetBaddataEject());
		m_CCProperty.SetNodataDetect(pCCOptionParam->GetNodataDetect());
		m_CCProperty.SetNodataCancel(pCCOptionParam->GetNodataCancel());
		m_CCProperty.SetNodataEject(pCCOptionParam->GetNodataEject());
	}
}

// Get scanning option settings
void CApp::GetOptionParam(COptionParam* pCCOptionParam)
{
	if(pCCOptionParam != NULL)
	{
		pCCOptionParam->SetPaperMisInsertionDetect(m_CCProperty.GetPaperMisInsertionDetect());
		pCCOptionParam->SetPaperMisInsertionCancel(m_CCProperty.GetPaperMisInsertionCancel());
		pCCOptionParam->SetPaperMisInsertionEject(m_CCProperty.GetPaperMisInsertionEject());
		pCCOptionParam->SetNoiseDetect(m_CCProperty.GetNoiseDetect());
		pCCOptionParam->SetNoiseCancel(m_CCProperty.GetNoiseCancel());
		pCCOptionParam->SetNoiseEject(m_CCProperty.GetNoiseEject());
		pCCOptionParam->SetDoubleFeedDetect(m_CCProperty.GetDoubleFeedDetect());
		pCCOptionParam->SetDoubleFeedCancel(m_CCProperty.GetDoubleFeedCancel());
		pCCOptionParam->SetDoubleFeedEject(m_CCProperty.GetDoubleFeedEject());
		pCCOptionParam->SetBaddataDetect(m_CCProperty.GetBaddataDetect());
		pCCOptionParam->SetBaddataCancel(m_CCProperty.GetBaddataCancel());
		pCCOptionParam->SetBaddataEject(m_CCProperty.GetBaddataEject());
		pCCOptionParam->SetNodataDetect(m_CCProperty.GetNodataDetect());
		pCCOptionParam->SetNodataCancel(m_CCProperty.GetNodataCancel());
		pCCOptionParam->SetNodataEject(m_CCProperty.GetNodataEject());
	}
}

// Regsiter callback function for barcode recognition
void CApp::SetBarcodeCallback(BARCODE_BACK_FUNC pBarcodeCB)
{
	m_pFuncBarcodeCB = pBarcodeCB;
}

// Register callback function for device status
void CApp::SetDeviceStatusCallback(DEVICE_STATUS_BACK_FUNC pDeviceStatusCB)
{
	m_pFuncDeviceStatusCB = pDeviceStatusCB;
}

// Register callback function for ink status
void CApp::SetInkStatusCallback(INK_STATUS_BACK_FUNC pInkStatusCB)
{
	m_pFuncInkStatusCB = pInkStatusCB;
}

// Obtain barcode data
int CApp::GetBarcode(CBarcodeResult *pCCBarcodeResult)
{
	return m_CCBarcode.GetBarcode(pCCBarcodeResult);
}

// Get device status
int CApp::GetDeviceStatusFunction(CDeviceStatus *pCCDeviceStatus)
{
	return m_CCInfo.GetDeviceStatusFunction(pCCDeviceStatus);
}

// Get ink status
int CApp::GetInkStatusFunction(CInkStatus* pCCInkStatus)
{
	return m_CCInfo.GetInkStatusFunction(pCCInkStatus);
}

// Get device information
int CApp::GetDeviceInfo(CDeviceInfo* pCCDeviceInfo)
{
	return m_CCInfo.GetDeviceInfoFunction(pCCDeviceInfo);
}

// Get maintenace counters
int CApp::GetMaintenanceCounter(CMaintenanceCounter* pCCMaintenanceCounter)
{
	return m_CCInfo.GetMaintenanceCounterFunction(pCCMaintenanceCounter);
}

// Callback function for notification of device status
int CApp::cbStatus(DWORD dwStatus)
{
	return g_pCCApp->Status(dwStatus);
}

int CApp::Status(DWORD dwStatus)
{
	if(m_pFuncDeviceStatusCB != NULL)
	{
		m_pFuncDeviceStatusCB();
	}

	return 0;
}

// Callback function for notification of ink status
int CApp::cbInkStatus(WORD wStatus)
{
	return g_pCCApp->InkStatus(wStatus);
}

int CApp::InkStatus(WORD wStatus)
{
	if(m_pFuncInkStatusCB != NULL)
	{
		m_pFuncInkStatusCB();
	}

	return 0;
}
